from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

dropbox_linked_team_application_added_tests: list[RuleTest] = [
    RuleTest(
        name="App linked for team is LOW severity",
        expected_result=True,
        log={
            "actor": {
                "_tag": "user",
                "user": {
                    "_tag": "team_member",
                    "account_id": "dbid:xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                    "display_name": "user_name",
                    "email": "user@domain.com",
                    "team_member_id": "dbmid:xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                },
            },
            "context": {"_tag": "team"},
            "details": {
                ".tag": "app_link_team_details",
                "app_info": {
                    ".tag": "team_linked_app",
                    "app_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                    "display_name": "dropbox-app-name",
                },
            },
            "event_category": {"_tag": "apps"},
            "event_type": {"_tag": "app_link_team", "description": "Linked app for team"},
            "involve_non_team_member": False,
            "origin": {
                "access_method": {".tag": "api", "request_id": "dbarod:xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"},
                "geo_location": {
                    "city": "Los Angeles",
                    "country": "US",
                    "ip_address": "1.2.3.4",
                    "region": "California",
                },
            },
            "timestamp": "2023-02-16 20:39:34",
        },
    ),
    RuleTest(
        name="A non-team linked event does not alert",
        expected_result=False,
        log={
            "actor": {
                "_tag": "user",
                "user": {
                    "_tag": "team_member",
                    "account_id": "dbid:xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                    "display_name": "user_name",
                    "email": "user@domain.com",
                    "team_member_id": "dbmid:xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                },
            },
            "context": {"_tag": "team"},
            "details": {
                ".tag": "app_link_member_details",
                "app_info": {
                    ".tag": "member_linked_app",
                    "app_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                    "display_name": "personal-dropbox-app-name",
                },
            },
            "event_category": {"_tag": "apps"},
            "event_type": {"_tag": "app_link_member", "description": "Linked app for member"},
            "involve_non_team_member": False,
            "origin": {
                "access_method": {".tag": "api", "request_id": "dbarod:xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"},
                "geo_location": {
                    "city": "Los Angeles",
                    "country": "US",
                    "ip_address": "1.2.3.4",
                    "region": "California",
                },
            },
            "timestamp": "2023-02-16 20:39:34",
        },
    ),
    RuleTest(
        name="App linked for team involving non-team member is HIGH severity",
        expected_result=True,
        log={
            "actor": {
                "_tag": "user",
                "user": {
                    "_tag": "team_member",
                    "account_id": "dbid:xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                    "display_name": "user_name",
                    "email": "user@domain.com",
                    "team_member_id": "dbmid:xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                },
            },
            "context": {"_tag": "team"},
            "details": {
                ".tag": "app_link_team_details",
                "app_info": {
                    ".tag": "team_linked_app",
                    "app_id": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                    "display_name": "dropbox-app-name",
                },
            },
            "event_category": {"_tag": "apps"},
            "event_type": {"_tag": "app_link_team", "description": "Linked app for team"},
            "involve_non_team_member": True,
            "origin": {
                "access_method": {".tag": "api", "request_id": "dbarod:xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"},
                "geo_location": {
                    "city": "Los Angeles",
                    "country": "US",
                    "ip_address": "1.2.3.4",
                    "region": "California",
                },
            },
            "timestamp": "2023-02-16 20:39:34",
        },
    ),
]


@panther_managed
class DropboxLinkedTeamApplicationAdded(Rule):
    default_description = "An application was linked to your Dropbox Account"
    display_name = "Dropbox Linked Team Application Added"
    default_reference = "https://help.dropbox.com/integrations/app-integrations"
    default_runbook = "Ensure that the application is valid and not malicious. Verify that this is expected. If not, determine other actions taken by this user recently and reach out to the user. If the event involved a non-team member, consider disabling the user's access while investigating.\n"
    default_severity = Severity.LOW
    tags = ["dropbox"]
    log_types = [LogType.DROPBOX_TEAM_EVENT]
    id = "Dropbox.Linked.Team.Application.Added-prototype"
    tests = dropbox_linked_team_application_added_tests

    def rule(self, event):
        return all(
            [
                deep_get(event, "event_type", "_tag", default="") == "app_link_team",
                deep_get(event, "event_type", "description", default="") == "Linked app for team",
            ],
        )

    def severity(self, event):
        # Anything involving non-team members should be High
        if event.get("involve_non_team_member", False):
            return "High"
        return "Low"

    def get_actor_type(self):
        # Admin who performed the action
        # Anonymous actor
        # Application that performed the action
        # Action performed by Dropbox
        # Action performed by reseller
        # User who performed the action
        return ("admin", "anonymous", "app", "dropbox", "reseller", "user")

    def title(self, event):
        # This will be one of the types returned by get_actor_type;
        # find the intersection and use that for the key
        actor_key = set(tuple(event.get("actor", {}).keys())).intersection(self.get_actor_type())
        if len(actor_key) == 1:
            display_name = deep_get(event, "actor", tuple(actor_key)[0], "display_name", default="<Unknown>")
        else:
            # Explicitly use "<Unknown>" if we find any length of keys != 1
            display_name = "<Unknown>"
        return f"Dropbox Team Member Linked App by [{display_name}]"

    def user_details(self, event):
        details = {}
        for actor_key, actor_value in event.get("actor", {}).items():
            if actor_key == "_tag":
                continue
            for user_key, user_info in actor_value.items():
                if user_key in ("_tag", "display_name"):
                    continue
                details[user_key] = user_info
        return details

    def alert_context(self, event):
        additional_user_details = self.user_details(event)
        return {
            "additional_user_details": additional_user_details,
            "app_display_name": deep_get(
                event,
                "details",
                "app_info",
                "display_name",
                default="<Unknown app display name>",
            ),
            "ip_address": deep_get(event, "origin", "geo_location", "ip_address", default="<Unknown IP address>"),
            "request_id": deep_get(event, "origin", "access_method", "request_id", default="<Unknown request ID>"),
        }



import argparse
import os
from typing import Tuple

from pypanther.display import print_rule_table, print_rules_as_json
from pypanther.get import get_panther_rules
from pypanther.import_main import NoMainModuleError, import_main
from pypanther.registry import registered_rules


def run(args: argparse.Namespace) -> Tuple[int, str]:
    rules = set()

    if not args.registered and not args.managed:
        return 1, "At least one of --registered or --managed is required"

    if args.registered:
        try:
            import_main(os.getcwd(), "main")
        except NoMainModuleError:
            return 1, "No main.py found. Cannot use --registered option without main.py."
        rules = registered_rules(
            log_types=args.log_types,
            id=args.id,
            create_alert=args.create_alert,
            dedup_period_minutes=args.dedup_period_minutes,
            display_name=args.display_name,
            enabled=args.enabled,
            summary_attributes=args.summary_attributes,
            threshold=args.threshold,
            tags=args.tags,
            default_severity=args.default_severity,
            default_description=args.default_description,
            default_reference=args.default_reference,
            default_runbook=args.default_runbook,
            default_destinations=args.default_destinations,
        )

    if args.managed:
        panther_rules = get_panther_rules(
            log_types=args.log_types,
            id=args.id,
            create_alert=args.create_alert,
            dedup_period_minutes=args.dedup_period_minutes,
            display_name=args.display_name,
            enabled=args.enabled,
            summary_attributes=args.summary_attributes,
            threshold=args.threshold,
            tags=args.tags,
            default_severity=args.default_severity,
            default_description=args.default_description,
            default_reference=args.default_reference,
            default_runbook=args.default_runbook,
            default_destinations=args.default_destinations,
        )
        for rule in panther_rules:
            rules.add(rule)

    try:
        match args.output:
            case "text":
                print_rule_table(list(rules), args.attributes)
            case "json":
                print_rules_as_json(list(rules), args.attributes)
            case _:
                return 1, f"Unsupported output: {args.output}"
    except AttributeError as err:
        return 1, f"Invalid attribute was given in --attributes option: {err!s}"

    return 0, ""



import argparse

API_DOCUMENTATION = "https://docs.panther.com/api-beta"


def for_public_api(parser: argparse.ArgumentParser, required: bool) -> None:
    parser.add_argument(
        "--api-token",
        type=str,
        help="The Panther API token to use. See: " + API_DOCUMENTATION,
        required=required,
    )

    parser.add_argument(
        "--api-host",
        type=str,
        help="The Panther API host to use. See: " + API_DOCUMENTATION,
        required=required,
    )



from difflib import SequenceMatcher

from pypanther import LogType, Rule, Severity, panther_managed


@panther_managed
class CiscoUmbrellaDNSFuzzyMatching(Rule):
    id = "CiscoUmbrella.DNS.FuzzyMatching-prototype"
    display_name = "Cisco Umbrella Domain Name Fuzzy Matching"
    enabled = False
    dedup_period_minutes = 15
    log_types = [LogType.CISCO_UMBRELLA_DNS]
    tags = ["Configuration Required", "DNS"]
    default_reference = "https://umbrella.cisco.com/blog/abcs-of-dns"
    default_severity = Severity.MEDIUM
    default_description = "Identify lookups to suspicious domains that could indicate a phishing attack."
    default_runbook = "Validate if your organization owns the domain, otherwise investigate the host that made the domain resolution.\n"
    DOMAIN = ""  # The domain to monitor for phishing, for example "google.com"
    # List all of your known-good domains here
    ALLOW_SET = {}
    SIMILARITY_RATIO = 0.7

    def rule(self, event):
        # Domains coming through umbrella end with a dot, such as google.com.
        domain = ".".join(event.get("domain").rstrip(".").split(".")[-2:]).lower()
        return (
            domain not in self.ALLOW_SET and SequenceMatcher(None, self.DOMAIN, domain).ratio() >= self.SIMILARITY_RATIO
        )

    def title(self, event):
        return f"Suspicious DNS resolution to {event.get('domain')}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

zoom_automatic_sign_out_disabled_tests: list[RuleTest] = [
    RuleTest(
        name="Automatic Signout Setting Disabled",
        expected_result=True,
        log={
            "action": "Update",
            "category_type": "Account",
            "operation_detail": "Security  - Automatically sign users out after a specified time: from On to Off",
            "operator": "example@example.io",
            "time": "2022-12-16 18:20:42",
        },
    ),
    RuleTest(
        name="Meeting Setting Disabled",
        expected_result=False,
        log={
            "action": "Update",
            "category_type": "Account",
            "operation_detail": "Security  - Require that all meetings are secured with one security option: from On to Off",
            "operator": "example@example.io",
            "time": "2022-12-16 18:15:38",
        },
    ),
]


@panther_managed
class ZoomAutomaticSignOutDisabled(Rule):
    default_description = "A Zoom User turned off your organization's setting to automatically sign users out after a specified period of time."
    display_name = "Zoom Automatic Sign Out Disabled"
    default_reference = "https://support.zoom.us/hc/en-us/articles/115005756143-Changing-account-security-settings#:~:text=Users%20need%20to%20sign%20in,of%205%20to%20120%20minutes"
    default_runbook = (
        "Confirm this user acted with valid business intent and determine whether this activity was authorized."
    )
    default_severity = Severity.MEDIUM
    log_types = [LogType.ZOOM_OPERATION]
    id = "Zoom.Automatic.Sign.Out.Disabled-prototype"
    tests = zoom_automatic_sign_out_disabled_tests

    def rule(self, event):
        operation_detail = event.get("operation_detail", "<NO_OPS_DETAIL>")
        operation_flag = "Automatically sign users out after a specified time: from On to Off"
        return (
            event.get("action", "<NO_ACTION>") == "Update"
            and event.get("category_type", "<NO_CATEGORY_TYPE>") == "Account"
            and (operation_flag in operation_detail)
        )

    def title(self, event):
        return f"Zoom User [{event.get('operator', '<NO_OPERATOR>')}] turned off your organization's setting to automatically sign users out after a specified time."



import argparse
import os
from typing import Tuple

from pypanther import display
from pypanther.get import get_panther_rules
from pypanther.import_main import NoMainModuleError, import_main
from pypanther.registry import registered_rules


def run(args: argparse.Namespace) -> Tuple[int, str]:
    try:
        import_main(os.getcwd(), "main")
    except NoMainModuleError:
        return 1, "No main.py found."
    found_rules = set(get_panther_rules(id=args.id)).union(registered_rules(id=args.id))
    if len(found_rules) == 0:
        return 1, f"Found no rules matching id={args.id}"
    if len(found_rules) > 1:
        return 1, f"Found multiple rules matching id={args.id}"
    rule = found_rules.pop()

    try:
        match args.output:
            case "text":
                display.print_rule_as_text(rule)
            case "json":
                display.print_rule_as_json(rule)
            case _:
                return 1, f"Unsupported output: {args.output}"
    except OSError as e:
        return 1, f"Error getting details for rule {args.id}: {e!r}"

    return 0, ""



import json
from unittest.mock import MagicMock

from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_mongodb_helpers import mongodb_alert_context

mongo_db_external_user_invited_tests: list[RuleTest] = [
    RuleTest(
        name="Internal Invite",
        expected_result=False,
        mocks=[RuleMock(object_name="ALLOWED_DOMAINS", return_value='[\n  "company.com"\n]')],
        log={
            "created": "2023-06-07 16:57:55",
            "currentValue": {},
            "eventTypeName": "INVITED_TO_ORG",
            "id": "6480b7139bd8a012345ABCDE",
            "isGlobalAdmin": False,
            "links": [
                {
                    "href": "https://cloud.mongodb.com/api/atlas/v1.0/orgs/12345xyzlmnce4f17d6e8e130/events/6480b7139bd8a012345ABCDE",
                    "rel": "self",
                },
            ],
            "orgId": "12345xyzlmnce4f17d6e8e130",
            "p_event_time": "2023-06-07 16:57:55",
            "p_log_type": "MongoDB.OrganizationEvent",
            "p_parse_time": "2023-06-07 17:04:42.59",
            "p_row_id": "ea276b16216684d9e198c0d0188a3d",
            "p_schema_version": 0,
            "p_source_id": "7c3cb124-9c30-492c-99e6-46518c232d73",
            "p_source_label": "MongoDB",
            "remoteAddress": "1.2.3.4",
            "targetUsername": "insider@company.com",
            "userId": "647f654f93bebc69123abc1",
            "username": "user@company.com",
        },
    ),
    RuleTest(
        name="External User Invite",
        expected_result=True,
        mocks=[RuleMock(object_name="ALLOWED_DOMAINS", return_value='[\n  "company.com"\n]')],
        log={
            "created": "2023-06-07 16:57:55",
            "currentValue": {},
            "eventTypeName": "INVITED_TO_ORG",
            "id": "6480b7139bd8a012345ABCDE",
            "isGlobalAdmin": False,
            "links": [
                {
                    "href": "https://cloud.mongodb.com/api/atlas/v1.0/orgs/12345xyzlmnce4f17d6e8e130/events/6480b7139bd8a012345ABCDE",
                    "rel": "self",
                },
            ],
            "orgId": "12345xyzlmnce4f17d6e8e130",
            "p_event_time": "2023-06-07 16:57:55",
            "p_log_type": "MongoDB.OrganizationEvent",
            "p_parse_time": "2023-06-07 17:04:42.59",
            "p_row_id": "ea276b16216684d9e198c0d0188a3d",
            "p_schema_version": 0,
            "p_source_id": "7c3cb124-9c30-492c-99e6-46518c232d73",
            "p_source_label": "MongoDB",
            "remoteAddress": "1.2.3.4",
            "targetUsername": "outsider@other.com",
            "userId": "647f654f93bebc69123abc1",
            "username": "user@company.com",
        },
    ),
]


@panther_managed
class MongoDBExternalUserInvited(Rule):
    default_description = "An external user has been invited to a MongoDB org. "
    display_name = "MongoDB External User Invited"
    default_severity = Severity.MEDIUM
    default_reference = "https://www.mongodb.com/docs/v4.2/tutorial/create-users/"
    tags = ["Configuration Required"]
    log_types = [LogType.MONGODB_ORGANIZATION_EVENT]
    id = "MongoDB.External.UserInvited-prototype"
    tests = mongo_db_external_user_invited_tests
    # Set domains allowed to join the organization ie. company.com
    ALLOWED_DOMAINS = []

    def rule(self, event):
        if isinstance(self.ALLOWED_DOMAINS, MagicMock):
            self.ALLOWED_DOMAINS = json.loads(self.ALLOWED_DOMAINS())  # pylint: disable=not-callable
        if deep_get(event, "eventTypeName", default="") == "INVITED_TO_ORG":
            target_user = deep_get(event, "targetUsername", default="")
            target_domain = target_user.split("@")[-1]
            return target_domain not in self.ALLOWED_DOMAINS
        return False

    def title(self, event):
        actor = event.get("username", "<USER_NOT_FOUND>")
        target = event.get("targetUsername", "<USER_NOT_FOUND>")
        org_id = event.get("orgId", "<ORG_NOT_FOUND>")
        return f"MongoDB Atlas: [{actor}] invited external user [{target}] to the org [{org_id}]"

    def alert_context(self, event):
        return mongodb_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import box_parse_additional_details, deep_get

box_shield_suspicious_alert_tests: list[RuleTest] = [
    RuleTest(
        name="Regular Event",
        expected_result=False,
        log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {"id": "12345678", "type": "user", "login": "ceo@example", "name": "Bob Cat"},
            "event_type": "DELETE",
        },
    ),
    RuleTest(
        name="Suspicious Login Event",
        expected_result=True,
        log={
            "type": "event",
            "additional_details": '{"shield_alert":{"rule_category":"Suspicious Locations","risk_score":60,"user":{"email":"bob@example"}}}',
            "created_by": {"id": "12345678", "type": "user", "login": "bob@example", "name": "Bob Cat"},
            "event_type": "SHIELD_ALERT",
            "source": {"id": "12345678", "type": "user"},
        },
    ),
    RuleTest(
        name="Suspicious Session Event",
        expected_result=True,
        log={
            "type": "event",
            "additional_details": '{"shield_alert":{"rule_category":"Suspicious Sessions","risk_score":70,"alert_summary":{"description":"First time in prior month user connected from ip 1.2.3.4."},"user":{"email":"bob@example"}}}',
            "created_by": {"id": "12345678", "type": "user", "login": "bob@example", "name": "Bob Cat"},
            "event_type": "SHIELD_ALERT",
            "source": {"id": "12345678", "type": "user"},
        },
    ),
    RuleTest(
        name="Suspicious Session Event - Low Risk",
        expected_result=False,
        log={
            "type": "event",
            "additional_details": '{"shield_alert":{"rule_category":"Suspicious Sessions","risk_score":10,"alert_summary":{"description":"First time in prior month user connected from ip 1.2.3.4."},"user":{"email":"bob@example"}}}',
            "created_by": {"id": "12345678", "type": "user", "login": "bob@example", "name": "Bob Cat"},
            "event_type": "SHIELD_ALERT",
            "source": {"id": "12345678", "type": "user"},
        },
    ),
]


@panther_managed
class BoxShieldSuspiciousAlert(Rule):
    id = "Box.Shield.Suspicious.Alert-prototype"
    display_name = "Box Shield Suspicious Alert Triggered"
    log_types = [LogType.BOX_EVENT]
    tags = ["Box", "Initial Access:Valid Accounts"]
    reports = {"MITRE ATT&CK": ["TA0001:T1078"]}
    default_severity = Severity.HIGH
    default_description = "A user login event or session event was tagged as medium to high severity by Box Shield.\n"
    default_reference = "https://developer.box.com/guides/events/shield-alert-events/"
    default_runbook = "Investigate whether this was triggered by an expected user event.\n"
    summary_attributes = ["event_type", "ip_address"]
    tests = box_shield_suspicious_alert_tests
    SUSPICIOUS_EVENT_TYPES = {"Suspicious Locations", "Suspicious Sessions"}

    def rule(self, event):
        if event.get("event_type") != "SHIELD_ALERT":
            return False
        alert_details = box_parse_additional_details(event).get("shield_alert", {})
        if alert_details.get("rule_category", "") in self.SUSPICIOUS_EVENT_TYPES:
            if alert_details.get("risk_score", 0) > 50:
                return True
        return False

    def title(self, event):
        details = box_parse_additional_details(event)
        description = deep_get(details, "shield_alert", "alert_summary", "description", default="")
        if description:
            return description
        return f"Shield medium to high risk, suspicious event alert triggered for user [{deep_get(details, 'shield_alert', 'user', 'email', default='<UNKNOWN_USER>')}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import filter_crowdstrike_fdr_event_type, get_crowdstrike_field

crowdstrike_dns_request_tests: list[RuleTest] = [
    RuleTest(
        name="Denylisted Domain",
        expected_result=True,
        log={
            "event_simpleName": "DnsRequest",
            "name": "DnsRequestMacV1",
            "aid": "00000000000000000000000000000001",
            "aip": "111.111.111.111",
            "cid": "00000000000000000000000000000002",
            "id": "11111111-0000-1111-0000-111111111111",
            "event_platform": "Mac",
            "timestamp": "2021-10-01 00:00:00.000Z",
            "ConfigBuild": "1007.4.0014301.11",
            "ConfigStateHash": "507116305",
            "Entitlements": "15",
            "ContextThreadId": "0",
            "ContextTimeStamp": "2021-10-08 19:55:04.448Z",
            "ContextProcessId": "111111111111111111",
            "EffectiveTransmissionClass": 2,
            "DomainName": "baddomain.com",
            "RequestType": "1",
            "p_event_time": "2021-10-08 19:55:04.448Z",
            "p_parse_time": "2021-10-08 20:09:41.933Z",
            "p_log_type": "Crowdstrike.DNSRequest",
            "p_row_id": "2ed00000000000000000000000000001",
            "p_source_id": "11111111-1111-1111-1111-111111111111",
            "p_source_label": "Crowdstrike",
            "p_any_ip_addresses": ["111.111.111.111"],
            "p_any_domain_names": ["baddomain.com"],
            "p_any_trace_ids": ["00000000000000000000000000000001", "00000000000000000000000000000002"],
        },
    ),
    RuleTest(
        name="Non-denylisted Domain",
        expected_result=False,
        log={
            "event_simpleName": "DnsRequest",
            "name": "DnsRequestMacV1",
            "aid": "00000000000000000000000000000001",
            "aip": "111.111.111.111",
            "cid": "00000000000000000000000000000002",
            "id": "11111111-0000-1111-0000-111111111111",
            "event_platform": "Mac",
            "timestamp": "2021-10-01 00:00:00.000Z",
            "ConfigBuild": "1007.4.0014301.11",
            "ConfigStateHash": "507116305",
            "Entitlements": "15",
            "ContextThreadId": "0",
            "ContextTimeStamp": "2021-10-08 19:55:04.448Z",
            "ContextProcessId": "111111111111111111",
            "EffectiveTransmissionClass": 2,
            "DomainName": "gooddomain.com",
            "RequestType": "1",
            "p_event_time": "2021-10-08 19:55:04.448Z",
            "p_parse_time": "2021-10-08 20:09:41.933Z",
            "p_log_type": "Crowdstrike.DNSRequest",
            "p_row_id": "2ed00000000000000000000000000001",
            "p_source_id": "11111111-1111-1111-1111-111111111111",
            "p_source_label": "Crowdstrike",
            "p_any_ip_addresses": ["111.111.111.111"],
            "p_any_domain_names": ["gooddomain.com"],
            "p_any_trace_ids": ["00000000000000000000000000000001", "00000000000000000000000000000002"],
        },
    ),
    RuleTest(
        name="Denylisted Domain (FDREvent)",
        expected_result=True,
        log={
            "aid": "307dc41ce39744f060622095f2805249",
            "aip": "10.0.0.0",
            "cid": "0cfb1a68ef6b49fdb0d2b12725057057",
            "ConfigBuild": "1007.4.0010306.1",
            "ConfigStateHash": "156025532",
            "ContextProcessId": "289977812183778042",
            "ContextThreadId": "0",
            "ContextTimestamp": "2020-05-24 23:50:06.989",
            "Entitlements": "15",
            "event": {
                "ConfigBuild": "1007.4.0010306.1",
                "ConfigStateHash": "156025532",
                "ContextProcessId": "289977812183778042",
                "ContextThreadId": "0",
                "ContextTimeStamp": "1590364206.989",
                "DomainName": "baddomain.com",
                "Entitlements": "15",
                "RequestType": "1",
                "aid": "307dc41ce39744f060622095f2805249",
                "aip": "10.0.0.0",
                "cid": "0cfb1a68ef6b49fdb0d2b12725057057",
                "event_platform": "Mac",
                "event_simpleName": "DnsRequest",
                "id": "4be06eb8-9e19-11ea-a7b0-026c15f3d8ed",
                "name": "DnsRequestMacV1",
                "timestamp": "1590364207259",
            },
            "event_platform": "Mac",
            "event_simplename": "DnsRequest",
            "fdr_event_type": "DnsRequest",
            "id": "4be06eb8-9e19-11ea-a7b0-026c15f3d8ed",
            "name": "DnsRequestMacV1",
            "p_any_domain_names": ["baddomain.com"],
            "p_any_ip_addresses": ["10.0.0.0"],
            "p_any_md5_hashes": ["0cfb1a68ef6b49fdb0d2b12725057057", "307dc41ce39744f060622095f2805249"],
            "p_any_trace_ids": ["0cfb1a68ef6b49fdb0d2b12725057057", "307dc41ce39744f060622095f2805249"],
            "p_event_time": "2020-05-24 23:50:06.989",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_parse_time": "2023-01-26 12:17:58.141",
            "p_row_id": "a21b385f60c08898ae918c84162d",
            "p_schema_version": 0,
            "timestamp": "2020-05-24 23:50:07.259",
        },
    ),
    RuleTest(
        name="Non-denylisted Domain (FDREvent)",
        expected_result=False,
        log={
            "aid": "307dc41ce39744f060622095f2805249",
            "aip": "10.0.0.0",
            "cid": "0cfb1a68ef6b49fdb0d2b12725057057",
            "ConfigBuild": "1007.4.0010306.1",
            "ConfigStateHash": "156025532",
            "ContextProcessId": "289977812183778042",
            "ContextThreadId": "0",
            "ContextTimeStamp": "2020-05-24 23:50:06.989",
            "Entitlements": "15",
            "event": {
                "ConfigBuild": "1007.4.0010306.1",
                "ConfigStateHash": "156025532",
                "ContextProcessId": "289977812183778042",
                "ContextThreadId": "0",
                "ContextTimeStamp": "1590364206.989",
                "DomainName": "gooddomain.com",
                "Entitlements": "15",
                "RequestType": "1",
                "aid": "307dc41ce39744f060622095f2805249",
                "aip": "10.0.0.0",
                "cid": "0cfb1a68ef6b49fdb0d2b12725057057",
                "event_platform": "Mac",
                "event_simpleName": "DnsRequest",
                "id": "4be06eb8-9e19-11ea-a7b0-026c15f3d8ed",
                "name": "DnsRequestMacV1",
                "timestamp": "1590364207259",
            },
            "event_platform": "Mac",
            "event_simplename": "DnsRequest",
            "fdr_event_type": "DnsRequest",
            "id": "4be06eb8-9e19-11ea-a7b0-026c15f3d8ed",
            "name": "DnsRequestMacV1",
            "p_any_domain_names": ["gooddomain.com"],
            "p_any_ip_addresses": ["10.0.0.0"],
            "p_any_md5_hashes": ["0cfb1a68ef6b49fdb0d2b12725057057", "307dc41ce39744f060622095f2805249"],
            "p_any_trace_ids": ["0cfb1a68ef6b49fdb0d2b12725057057", "307dc41ce39744f060622095f2805249"],
            "p_event_time": "2020-05-24 23:50:06.989",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_parse_time": "2023-01-26 12:17:58.141",
            "p_row_id": "a21b385f60c08898ae918c84162d",
            "p_schema_version": 0,
            "timestamp": "2020-05-24 23:50:07.259",
        },
    ),
    RuleTest(
        name="Denylisted Domain (but Non-DNS type) (FDREvent)",
        expected_result=False,
        log={
            "event_simpleName": "Event_DetectionSummaryEvent",
            "name": "DnsRequestMacV1",
            "aid": "00000000000000000000000000000001",
            "aip": "111.111.111.111",
            "cid": "00000000000000000000000000000002",
            "id": "11111111-0000-1111-0000-111111111111",
            "event": {
                "aid": "00000000000000000000000000000001",
                "event_simpleName": "Event_DetectionSummaryEvent",
                "name": "DnsRequestMacV1",
                "aip": "111.111.111.111",
                "cid": "00000000000000000000000000000002",
                "id": "11111111-0000-1111-0000-111111111111",
                "event_platform": "Mac",
                "timestamp": "2021-10-01 00:00:00.000Z",
                "ConfigBuild": "1007.4.0014301.11",
                "ConfigStateHash": "507116305",
                "Entitlements": "15",
                "ContextThreadId": "0",
                "ContextTimeStamp": "2021-10-08 19:55:04.448Z",
                "ContextProcessId": "111111111111111111",
                "EffectiveTransmissionClass": 2,
                "DomainName": "baddomain.com",
                "RequestType": "1",
            },
            "event_platform": "Mac",
            "fdr_event_type": "Event_DetectionSummaryEvent",
            "timestamp": "2021-10-01 00:00:00.000Z",
            "ConfigBuild": "1007.4.0014301.11",
            "ConfigStateHash": "507116305",
            "Entitlements": "15",
            "ContextThreadId": "0",
            "ContextTimeStamp": "2021-10-08 19:55:04.448Z",
            "ContextProcessId": "111111111111111111",
            "EffectiveTransmissionClass": 2,
            "RequestType": "1",
            "p_event_time": "2021-10-08 19:55:04.448Z",
            "p_parse_time": "2021-10-08 20:09:41.933Z",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_row_id": "2ed00000000000000000000000000001",
            "p_source_id": "11111111-1111-1111-1111-111111111111",
            "p_source_label": "Crowdstrike",
            "p_any_ip_addresses": ["111.111.111.111"],
            "p_any_domain_names": ["baddomain.com"],
            "p_any_trace_ids": ["00000000000000000000000000000001", "00000000000000000000000000000002"],
        },
    ),
]


@panther_managed
class CrowdstrikeDNSRequest(Rule):
    id = "Crowdstrike.DNS.Request-prototype"
    display_name = "DNS request to denylisted domain"
    enabled = False
    log_types = [LogType.CROWDSTRIKE_DNS_REQUEST, LogType.CROWDSTRIKE_FDR_EVENT]
    tags = ["Crowdstrike", "Initial Access:Phishing", "Configuration Required"]
    default_severity = Severity.CRITICAL
    reports = {"MITRE ATT&CK": ["TA0001:T1566"]}
    default_description = "A DNS request was made to a domain on an explicit denylist"
    default_reference = "https://docs.runpanther.io/data-onboarding/supported-logs/crowdstrike#crowdstrike-dnsrequest"
    default_runbook = "Filter for host ID in title in Crowdstrike Host Management console to identify the system that queried the domain."
    dedup_period_minutes = 15
    summary_attributes = ["DomainName", "aid", "p_any_domain_names", "p_any_ip_addresses"]
    tests = crowdstrike_dns_request_tests
    # baddomain.com is present for testing purposes. Add domains you wish to be alerted on to this list
    DENYLIST = ["baddomain.com"]

    def rule(self, event):
        # We need to run either for Crowdstrike.DnsRequest or for Crowdstrike.FDREvent with the
        # 'DnsRequest' fdr_event_type. Crowdstrike.DnsRequest is covered because of the
        # association with the type
        if filter_crowdstrike_fdr_event_type(event, "DnsRequest"):
            return False
        if get_crowdstrike_field(event, "DomainName") in self.DENYLIST:
            return True
        return False

    def title(self, event):
        return (
            f"A denylisted domain [{get_crowdstrike_field(event, 'DomainName')}] was "
            + f"queried by host {event.get('aid')}"
        )

    def dedup(self, event):
        #  Alert on every individual lookup of a bad domain, per machine
        return f"{get_crowdstrike_field(event, 'DomainName')}-{event.get('aid')}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp_base_helpers import get_k8s_info
from pypanther.helpers.gcp_environment import PRODUCTION_PROJECT_IDS, rule_exceptions
from pypanther.helpers.panther_base_helpers import deep_walk

gcpk8s_exec_into_pod_tests: list[RuleTest] = [
    RuleTest(
        name="Allowed User",
        expected_result=False,
        log={
            "protoPayload": {
                "authenticationInfo": {
                    "principalEmail": "system:serviceaccount:example-namespace:example-namespace-service-account",
                },
                "authorizationInfo": [
                    {
                        "permission": "io.k8s.core.v1.pods.exec.create",
                        "resource": "core/v1/namespaces/opa/pods/opa-57998cf7c5-bjkfk/exec",
                    },
                ],
                "methodName": "io.k8s.core.v1.pods.exec.create",
                "requestMetadata": {
                    "callerIp": "88.88.88.88",
                    "callerSuppliedUserAgent": "kubectl/v1.40.8 (darwin/amd64) kubernetes/6575935",
                },
                "resourceName": "core/v1/namespaces/example/pods/one-off-46666967280/exec",
                "timestamp": "2022-03-04T16:01:49.978756Z",
            },
            "resource": {"type": "k8s_cluster", "labels": {"project_id": "rigup-production"}},
        },
    ),
    RuleTest(
        name="Disallowed User",
        expected_result=True,
        log={
            "protoPayload": {
                "authenticationInfo": {"principalEmail": "disallowed.user@example.com"},
                "authorizationInfo": [
                    {
                        "permission": "io.k8s.core.v1.pods.exec.create",
                        "resource": "core/v1/namespaces/example/pods/example-57998cf7c5-bjkfk/exec",
                    },
                ],
                "methodName": "io.k8s.core.v1.pods.exec.create",
                "requestMetadata": {
                    "callerIp": "88.88.88.88",
                    "callerSuppliedUserAgent": "kubectl/v1.40.8 (darwin/amd64) kubernetes/6575935",
                },
                "resourceName": "core/v1/namespaces/example/pods/one-off-valerii-tovstyk-1646666967280/exec",
                "timestamp": "2022-03-04T16:01:49.978756Z",
            },
            "resource": {"type": "k8s_cluster", "labels": {"project_id": "rigup-production"}},
        },
    ),
    RuleTest(
        name="Disallowed User2 - not an allowed namespace",
        expected_result=True,
        log={
            "protoPayload": {
                "authenticationInfo": {"principalEmail": "example-allowed-user@example.com"},
                "authorizationInfo": [
                    {
                        "permission": "io.k8s.core.v1.pods.exec.create",
                        "resource": "core/v1/namespaces/istio-system/pods/opa-57998cf7c5-bjkfk/exec",
                    },
                ],
                "methodName": "io.k8s.core.v1.pods.exec.create",
                "requestMetadata": {
                    "callerIp": "88.88.88.88",
                    "callerSuppliedUserAgent": "kubectl/v1.40.8 (darwin/amd64) kubernetes/6575935",
                },
                "resourceName": "core/v1/namespaces/istio-system/pods/one-off-valerii-tovstyk-1646666967280/exec",
                "timestamp": "2022-03-04T16:01:49.978756Z",
            },
            "resource": {"type": "k8s_cluster", "labels": {"project_id": "rigup-production"}},
        },
    ),
]


@panther_managed
class GCPK8sExecIntoPod(Rule):
    id = "GCP.K8s.ExecIntoPod-prototype"
    display_name = "Exec into Pod"
    enabled = False
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["GCP", "Security Control", "Configuration Required"]
    default_severity = Severity.MEDIUM
    default_description = "Alerts when users exec into pod. Possible to specify specific projects and allowed users.\n"
    default_runbook = "Investigate the user and determine why. Advise that it is discouraged practice. Create ticket if appropriate.\n"
    default_reference = "https://cloud.google.com/migrate/containers/docs/troubleshooting/executing-shell-commands"
    tests = gcpk8s_exec_into_pod_tests

    def rule(self, event):
        # Defaults to False (no alert) unless method is exec and principal not allowed
        if not all(
            [
                deep_walk(event, "protoPayload", "methodName") == "io.k8s.core.v1.pods.exec.create",
                deep_walk(event, "resource", "type") == "k8s_cluster",
            ],
        ):
            return False
        k8s_info = get_k8s_info(event)
        principal = deep_walk(k8s_info, "principal", default="<NO PRINCIPAL>")
        namespace = deep_walk(k8s_info, "namespace", default="<NO NAMESPACE>")
        project_id = deep_walk(k8s_info, "project_id", default="<NO PROJECT_ID>")
        # rule_exceptions that are allowed temporarily are defined in gcp_environment.py
        # Some execs have principal which is long numerical UUID, appears to be k8s internals
        for allowed_principal in deep_walk(rule_exceptions, "gcp_k8s_exec_into_pod", "allowed_principals", default=[]):
            allowed_principals = deep_walk(allowed_principal, "principals", default=[])
            allowed_namespaces = deep_walk(allowed_principal, "namespaces", default=[])
            allowed_project_ids = deep_walk(allowed_principal, "projects", default=[])
            if (
                principal in allowed_principals
                and (namespace in allowed_namespaces or allowed_namespaces == [])
                and (project_id in allowed_project_ids or allowed_project_ids == [])
            ):
                if "@" not in principal:
                    return False
        return True

    def severity(self, event):
        project_id = deep_walk(get_k8s_info(event), "project_id", default="<NO PROJECT_ID>")
        if project_id in PRODUCTION_PROJECT_IDS:
            return "high"
        return "info"

    def title(self, event):
        # TODO: use unified data model field in title for actor
        k8s_info = get_k8s_info(event)
        principal = deep_walk(k8s_info, "principal", default="<NO PRINCIPAL>")
        project_id = deep_walk(k8s_info, "project_id", default="")
        pod = deep_walk(k8s_info, "pod", default="")
        namespace = deep_walk(k8s_info, "namespace", default="")
        return f"Exec into pod namespace/{namespace}/pod/{pod} by {principal} in {project_id}"

    def alert_context(self, event):
        return get_k8s_info(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

github_repo_created_tests: list[RuleTest] = [
    RuleTest(
        name="GitHub - Repo Created",
        expected_result=True,
        log={
            "actor": "cat",
            "action": "repo.create",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
        },
    ),
    RuleTest(
        name="GitHub - Repo Archived",
        expected_result=False,
        log={
            "actor": "cat",
            "action": "repo.archived",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
        },
    ),
]


@panther_managed
class GithubRepoCreated(Rule):
    id = "Github.Repo.Created-prototype"
    display_name = "GitHub Repository Created"
    log_types = [LogType.GITHUB_AUDIT]
    tags = ["GitHub"]
    default_reference = "https://docs.github.com/en/get-started/quickstart/create-a-repo"
    default_severity = Severity.INFO
    default_description = "Detects when a repository is created."
    tests = github_repo_created_tests

    def rule(self, event):
        return event.get("action") == "repo.create"

    def title(self, event):
        return f"Repository [{event.get('repo', '<UNKNOWN_REPO>')}] created."



from pypanther.base import DataModel, DataModelMapping, LogType


class StandardOCSFNetworkActivity(DataModel):
    id: str = "Standard.OCSF.NetworkActivity"
    display_name: str = "OCSF Network Activity"
    enabled: bool = True
    log_types: list[str] = [LogType.OCSF_NETWORK_ACTIVITY]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="destination_ip", path="$.dst_endpoint.ip"),
        DataModelMapping(name="destination_port", path="$.dst_endpoint.port"),
        DataModelMapping(name="source_ip", path="$.src_endpoint.ip"),
        DataModelMapping(name="source_port", path="$.src_endpoint.port"),
        DataModelMapping(name="log_status", path="status_code"),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context
from pypanther.helpers.panther_default import aws_cloudtrail_success

aws_config_service_created_tests: list[RuleTest] = [
    RuleTest(
        name="Config Recorder Delivery Channel Created",
        expected_result=True,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                    "webIdFederationData": {},
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "config.amazonaws.com",
            "eventName": "PutDeliveryChannel",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {"configurationRecorderName": "default"},
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="Config Recorder Deleted",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                    "webIdFederationData": {},
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "config.amazonaws.com",
            "eventName": "DeleteDeliveryChannel",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {"configurationRecorderName": "default"},
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="Error Creating Config Recorder Delivery Channel",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "errorCode": "InvalidDeliveryChannelNameException",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                    "webIdFederationData": {},
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "config.amazonaws.com",
            "eventName": "PutDeliveryChannel",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {"configurationRecorderName": "default"},
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
]


@panther_managed
class AWSConfigServiceCreated(Rule):
    id = "AWS.ConfigService.Created-prototype"
    display_name = "AWS Config Service Created"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Security Control", "Discovery:Cloud Service Discovery"]
    reports = {"CIS": ["3.9"], "MITRE ATT&CK": ["TA0007:T1526"]}
    default_severity = Severity.INFO
    default_description = "An AWS Config Recorder or Delivery Channel was created\n"
    default_runbook = "Verify that the Config Service changes were authorized. If not, revert them and investigate who caused the change. Consider altering permissions to prevent this from happening again in the future.\n"
    default_reference = "https://aws.amazon.com/config/"
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    tests = aws_config_service_created_tests
    # API calls that are indicative of an AWS Config Service change
    CONFIG_SERVICE_CREATE_EVENTS = {"PutDeliveryChannel", "PutConfigurationRecorder", "StartConfigurationRecorder"}

    def rule(self, event):
        return aws_cloudtrail_success(event) and event.get("eventName") in self.CONFIG_SERVICE_CREATE_EVENTS

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get

gcpia_mservice_accountsget_access_token_privilege_escalation_tests: list[RuleTest] = [
    RuleTest(
        name="iam.serviceAccounts.getAccessToken granted",
        expected_result=True,
        log={
            "protoPayload": {
                "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "status": {},
                "authenticationInfo": {
                    "principalEmail": "some-project@company.iam.gserviceaccount.com",
                    "serviceAccountKeyName": "//iam.googleapis.com/projects/some-project/serviceAccounts/some-project@company.iam.gserviceaccount.com/keys/a378358365ff3d22e9c1a72fecf4605ddff76b47",
                    "principalSubject": "serviceAccount:some-project@company.iam.gserviceaccount.com",
                },
                "requestMetadata": {
                    "callerIp": "1.2.3.4",
                    "requestAttributes": {"time": "2024-02-26T17:15:16.327542536Z", "auth": {}},
                    "destinationAttributes": {},
                },
                "serviceName": "iamcredentials.googleapis.com",
                "methodName": "SignJwt",
                "authorizationInfo": [
                    {"permission": "iam.serviceAccounts.getAccessToken", "granted": True, "resourceAttributes": {}},
                ],
                "resourceName": "projects/-/serviceAccounts/114885146936855121342",
                "request": {
                    "name": "projects/-/serviceAccounts/some-project@company.iam.gserviceaccount.com",
                    "@type": "type.googleapis.com/google.iam.credentials.v1.SignJwtRequest",
                },
            },
            "insertId": "1hu88qbef4d2o",
            "resource": {
                "type": "service_account",
                "labels": {
                    "project_id": "some-project",
                    "unique_id": "114885146936855121342",
                    "email_id": "some-project@company.iam.gserviceaccount.com",
                },
            },
            "timestamp": "2024-02-26T17:15:16.314854637Z",
            "severity": "INFO",
            "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Fdata_access",
            "receiveTimestamp": "2024-02-26T17:15:17.100020459Z",
        },
    ),
    RuleTest(
        name="iam.serviceAccounts.getAccessToken not granted",
        expected_result=False,
        log={
            "protoPayload": {
                "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "status": {},
                "authenticationInfo": {
                    "principalEmail": "some-project@company.iam.gserviceaccount.com",
                    "serviceAccountKeyName": "//iam.googleapis.com/projects/some-project/serviceAccounts/some-project@company.iam.gserviceaccount.com/keys/a378358365ff3d22e9c1a72fecf4605ddff76b47",
                    "principalSubject": "serviceAccount:some-project@company.iam.gserviceaccount.com",
                },
                "requestMetadata": {
                    "callerIp": "1.2.3.4",
                    "requestAttributes": {"time": "2024-02-26T17:15:16.327542536Z", "auth": {}},
                    "destinationAttributes": {},
                },
                "serviceName": "iamcredentials.googleapis.com",
                "methodName": "SignJwt",
                "authorizationInfo": [
                    {"permission": "iam.serviceAccounts.getAccessToken", "granted": False, "resourceAttributes": {}},
                ],
                "resourceName": "projects/-/serviceAccounts/114885146936855121342",
                "request": {
                    "name": "projects/-/serviceAccounts/some-project@company.iam.gserviceaccount.com",
                    "@type": "type.googleapis.com/google.iam.credentials.v1.SignJwtRequest",
                },
            },
            "insertId": "1hu88qbef4d2o",
            "resource": {
                "type": "service_account",
                "labels": {
                    "project_id": "some-project",
                    "unique_id": "114885146936855121342",
                    "email_id": "some-project@company.iam.gserviceaccount.com",
                },
            },
            "timestamp": "2024-02-26T17:15:16.314854637Z",
            "severity": "INFO",
            "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Fdata_access",
            "receiveTimestamp": "2024-02-26T17:15:17.100020459Z",
        },
    ),
]


@panther_managed
class GCPIAMserviceAccountsgetAccessTokenPrivilegeEscalation(Rule):
    id = "GCP.IAM.serviceAccounts.getAccessToken.Privilege.Escalation-prototype"
    display_name = "GCP IAM serviceAccounts getAccessToken Privilege Escalation"
    log_types = [LogType.GCP_AUDIT_LOG]
    reports = {"MITRE ATT&CK": ["TA0004:T1548"]}
    default_severity = Severity.HIGH
    default_description = "The Identity and Access Management (IAM) service manages authorization and authentication for a GCP environment. This means that there are very likely multiple privilege escalation methods that use the IAM service and/or its permissions."
    default_reference = "https://rhinosecuritylabs.com/gcp/privilege-escalation-google-cloud-platform-part-1/"
    tests = gcpia_mservice_accountsget_access_token_privilege_escalation_tests

    def rule(self, event):
        authorization_info = event.deep_walk("protoPayload", "authorizationInfo")
        if not authorization_info:
            return False
        for auth in authorization_info:
            if auth.get("permission") == "iam.serviceAccounts.getAccessToken" and auth.get("granted") is True:
                return True
        return False

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        operation = deep_get(event, "protoPayload", "methodName", default="<OPERATION_NOT_FOUND>")
        project_id = deep_get(event, "resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: [{actor}] performed [{operation}] on project [{project_id}]"

    def alert_context(self, event):
        return gcp_alert_context(event)



from pypanther import LogType, Rule, Severity, panther_managed


@panther_managed
class AWSConsoleLogin(Rule):
    id = "AWS.Console.Login-prototype"
    display_name = "AWS Console Login"
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.INFO
    create_alert = False

    def rule(self, event):
        return event.get("eventName") == "ConsoleLogin"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

push_security_mfa_method_changed_tests: list[RuleTest] = [
    RuleTest(
        name="All MFA methods removed",
        expected_result=True,
        log={
            "id": "d1e5794f-666d-4cba-abae-c6d889ca1903",
            "new": {
                "appId": "67ef5c13-b5e6-4945-af7b-c11ac98f630f",
                "appType": "CONTENTFUL",
                "creationTimestamp": 1707775048.0,
                "email": "jet.black@issp.com",
                "employeeId": "ca6cf7ce-90e6-4eb5-a262-7899bc48c39c",
                "id": "5e15ce4c-6b93-4fbf-aed9-1890775efa90",
                "lastUsedTimestamp": None,
                "loginMethods": {"oidcLogin": None, "oktaSwaLogin": False, "passwordLogin": False, "samlLogin": None},
                "mfaMethods": [],
                "mfaRegistered": False,
                "passwordId": None,
            },
            "object": "ACCOUNT",
            "old": {
                "appId": "67ef5c13-b5e6-4945-af7b-c11ac98f630f",
                "appType": "CONTENTFUL",
                "creationTimestamp": 1707775048.0,
                "email": "jet.black@issp.com",
                "employeeId": "ca6cf7ce-90e6-4eb5-a262-7899bc48c39c",
                "id": "5e15ce4c-6b93-4fbf-aed9-1890775efa90",
                "lastUsedTimestamp": None,
                "loginMethods": {"oidcLogin": None, "oktaSwaLogin": False, "passwordLogin": False, "samlLogin": None},
                "mfaMethods": ["SMS"],
                "mfaRegistered": False,
                "passwordId": None,
            },
            "timestamp": 1707775049.0,
            "type": "CREATE",
            "version": "1",
        },
    ),
    RuleTest(
        name="First seen",
        expected_result=False,
        log={
            "id": "d1e5794f-666d-4cba-abae-c6d889ca1903",
            "new": {
                "appId": "67ef5c13-b5e6-4945-af7b-c11ac98f630f",
                "appType": "CONTENTFUL",
                "creationTimestamp": 1707775048.0,
                "email": "jet.black@issp.com",
                "employeeId": "ca6cf7ce-90e6-4eb5-a262-7899bc48c39c",
                "id": "5e15ce4c-6b93-4fbf-aed9-1890775efa90",
                "lastUsedTimestamp": None,
                "loginMethods": {"oidcLogin": None, "oktaSwaLogin": False, "passwordLogin": False, "samlLogin": None},
                "mfaMethods": ["SMS", "APP_OTP"],
                "mfaRegistered": False,
                "passwordId": None,
            },
            "object": "ACCOUNT",
            "old": None,
            "timestamp": 1707775049.0,
            "type": "CREATE",
            "version": "1",
        },
    ),
    RuleTest(
        name="MFA method added",
        expected_result=True,
        log={
            "id": "d1e5794f-666d-4cba-abae-c6d889ca1903",
            "new": {
                "appId": "67ef5c13-b5e6-4945-af7b-c11ac98f630f",
                "appType": "CONTENTFUL",
                "creationTimestamp": 1707775048.0,
                "email": "jet.black@issp.com",
                "employeeId": "ca6cf7ce-90e6-4eb5-a262-7899bc48c39c",
                "id": "5e15ce4c-6b93-4fbf-aed9-1890775efa90",
                "lastUsedTimestamp": None,
                "loginMethods": {"oidcLogin": None, "oktaSwaLogin": False, "passwordLogin": False, "samlLogin": None},
                "mfaMethods": ["SMS", "APP_OTP"],
                "mfaRegistered": False,
                "passwordId": None,
            },
            "object": "ACCOUNT",
            "old": {
                "appId": "67ef5c13-b5e6-4945-af7b-c11ac98f630f",
                "appType": "CONTENTFUL",
                "creationTimestamp": 1707775048.0,
                "email": "jet.black@issp.com",
                "employeeId": "ca6cf7ce-90e6-4eb5-a262-7899bc48c39c",
                "id": "5e15ce4c-6b93-4fbf-aed9-1890775efa90",
                "lastUsedTimestamp": None,
                "loginMethods": {"oidcLogin": None, "oktaSwaLogin": False, "passwordLogin": False, "samlLogin": None},
                "mfaMethods": ["SMS"],
                "mfaRegistered": False,
                "passwordId": None,
            },
            "timestamp": 1707775049.0,
            "type": "CREATE",
            "version": "1",
        },
    ),
    RuleTest(
        name="No MFA method change",
        expected_result=False,
        log={
            "id": "d1e5794f-666d-4cba-abae-c6d889ca1903",
            "new": {
                "appId": "67ef5c13-b5e6-4945-af7b-c11ac98f630f",
                "appType": "CONTENTFUL",
                "creationTimestamp": 1707775048.0,
                "email": "jet.black@issp.com",
                "employeeId": "ca6cf7ce-90e6-4eb5-a262-7899bc48c39c",
                "id": "5e15ce4c-6b93-4fbf-aed9-1890775efa90",
                "lastUsedTimestamp": None,
                "loginMethods": {"oidcLogin": None, "oktaSwaLogin": False, "passwordLogin": False, "samlLogin": None},
                "mfaMethods": ["SMS", "APP_OTP"],
                "mfaRegistered": False,
                "passwordId": None,
            },
            "object": "ACCOUNT",
            "old": {
                "appId": "67ef5c13-b5e6-4945-af7b-c11ac98f630f",
                "appType": "CONTENTFUL",
                "creationTimestamp": 1707775048.0,
                "email": "jet.black@issp.com",
                "employeeId": "ca6cf7ce-90e6-4eb5-a262-7899bc48c39c",
                "id": "5e15ce4c-6b93-4fbf-aed9-1890775efa90",
                "lastUsedTimestamp": None,
                "loginMethods": {"oidcLogin": None, "oktaSwaLogin": False, "passwordLogin": False, "samlLogin": None},
                "mfaMethods": ["SMS", "APP_OTP"],
                "mfaRegistered": False,
                "passwordId": None,
            },
            "timestamp": 1707775049.0,
            "type": "CREATE",
            "version": "1",
        },
    ),
]


@panther_managed
class PushSecurityMFAMethodChanged(Rule):
    id = "Push.Security.MFA.Method.Changed-prototype"
    display_name = "Push Security SaaS App MFA Method Changed"
    log_types = [LogType.PUSH_SECURITY_ENTITIES]
    default_severity = Severity.INFO
    default_description = "MFA method on SaaS app changed"
    tests = push_security_mfa_method_changed_tests

    def rule(self, event):
        if event.get("object") != "ACCOUNT":
            return False
        if event.get("old") is None:
            return False
        new_mfa_methods = set(event.deep_get("new", "mfaMethods"))
        old_mfa_methods = set(event.deep_get("old", "mfaMethods", default=[]))
        if new_mfa_methods != old_mfa_methods:
            return True
        return False

    def severity(self, event):
        if event.deep_get("new", "mfaMethods") == []:
            return "HIGH"
        return "LOW"

    def title(self, event):
        mfa_methods = ", ".join(event.deep_get("new", "mfaMethods", default="No MFA"))
        new_email = event.deep_get("new", "email")
        new_apptype = event.deep_get("new", "appType")
        if mfa_methods == "":
            return f"{new_email} removed all MFA methods on {new_apptype}"
        return f"{new_email} changed MFA method to {mfa_methods} on {new_apptype}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

panther_detection_deleted_tests: list[RuleTest] = [
    RuleTest(
        name="Delete 1 Detection",
        expected_result=True,
        log={
            "actionName": "DELETE_DETECTION",
            "actionParams": {"dynamic": {"input": {"detections": [{"id": "GitHub.Team.Modified"}]}}},
            "actionResult": "SUCCEEDED",
            "actor": {
                "attributes": {"email": "homer@springfield.gov", "emailVerified": True, "roleId": "11111111"},
                "id": "1111111",
                "name": "Homer Simpson",
                "type": "USER",
            },
            "errors": None,
            "p_log_type": "Panther.Audit",
            "sourceIP": "1.2.3.4",
            "timestamp": "2022-04-28 15:30:22.42",
        },
    ),
    RuleTest(
        name="Delete Many Detections",
        expected_result=True,
        log={
            "actionName": "DELETE_DETECTION",
            "actionParams": {
                "dynamic": {
                    "input": {
                        "detections": [
                            {"id": "Github.Repo.Created"},
                            {"id": "Okta.Global.MFA.Disabled"},
                            {"id": "Okta.AdminRoleAssigned"},
                            {"id": "Okta.BruteForceLogins"},
                        ],
                    },
                },
            },
            "actionResult": "SUCCEEDED",
            "actor": {
                "attributes": {"email": "homer@springfield.gov", "emailVerified": True, "roleId": "111111"},
                "id": "1111111",
                "name": "Homer Simpson",
                "type": "USER",
            },
            "errors": None,
            "p_log_type": "Panther.Audit",
            "sourceIP": "1.2.3.4.",
            "timestamp": "2022-04-28 15:34:43.067",
        },
    ),
    RuleTest(
        name="Non-Delete event",
        expected_result=False,
        log={
            "actionName": "GET_GENERAL_SETTINGS",
            "actionParams": {},
            "actionResult": "SUCCEEDED",
            "actor": {
                "attributes": {"email": "homer@springfield.gov", "emailVerified": True, "roleId": "111111"},
                "id": "111111",
                "name": "Homer Simpson",
                "type": "USER",
            },
            "errors": None,
            "p_log_type": "Panther.Audit",
        },
    ),
]


@panther_managed
class PantherDetectionDeleted(Rule):
    id = "Panther.Detection.Deleted-prototype"
    display_name = "Detection content has been deleted from Panther"
    log_types = [LogType.PANTHER_AUDIT]
    default_severity = Severity.INFO
    tags = ["DataModel", "Defense Evasion:Impair Defenses"]
    reports = {"MITRE ATT&CK": ["TA0005:T1562"]}
    default_description = "Detection content has been removed from Panther."
    default_runbook = "Ensure this change was approved and appropriate."
    default_reference = "https://docs.panther.com/system-configuration/panther-audit-logs/querying-and-writing-detections-for-panther-audit-logs"
    summary_attributes = ["p_any_ip_addresses"]
    tests = panther_detection_deleted_tests
    PANTHER_DETECTION_DELETE_ACTIONS = [
        "DELETE_DATA_MODEL",
        "DELETE_DETECTION",
        "DELETE_DETECTION_PACK_SOURCE",
        "DELETE_GLOBAL_HELPER",
        "DELETE_LOOKUP_TABLE",
        "DELETE_SAVED_DATA_LAKE_QUERY",
    ]

    def rule(self, event):
        return (
            event.get("actionName") in self.PANTHER_DETECTION_DELETE_ACTIONS
            and event.get("actionResult") == "SUCCEEDED"
        )

    def title(self, event):
        return f"Detection Content has been deleted by {event.udm('actor_user')}"

    def alert_context(self, event):
        detections_list = deep_get(event, "actionParams", "dynamic", "input", "detections")
        if detections_list is None:
            detections_list = deep_get(event, "actionParams", "input", "detections", default=[])
        return {
            "deleted_detections_list": [x.get("id") for x in detections_list],
            "user": event.udm("actor_user"),
            "ip": event.udm("source_ip"),
        }



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context

awsiam_group_read_only_events_tests: list[RuleTest] = [
    RuleTest(
        name="Get Group",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "883efb94-aa58-4512-beb7-10a5fffa33e4",
            "eventName": "GetGroup",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2022-12-11 19:42:55",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": True,
            "recipientAccountId": "1231231234",
            "requestID": "f92dd1a7-ad07-4fef-9511-1081d2dd3585",
            "requestParameters": {"maxItems": 1000, "userName": "user-name"},
            "sourceIPAddress": "cloudformation.amazonaws.com",
            "userAgent": "cloudformation.amazonaws.com",
            "userIdentity": {
                "accessKeyId": "ASIAVKVYIOO7BDL4T5NG",
                "accountId": "1231231234",
                "arn": "arn:aws:sts::1231231234:assumed-role/AssumedRole-us-east-2/123123123456",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "AROAVKVYIOO7JN7TN7NSA:123123123456",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-12-11T19:42:54Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "1231231234",
                        "arn": "arn:aws:iam::1231231234:role/PAssumedRole-us-east-2",
                        "principalId": "AROAVKVYIOO7JN7TN7NSA",
                        "type": "Role",
                        "userName": "AssumedRole-us-east-2",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Get Group Policy",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "883efb94-aa58-4512-beb7-10a5fffa33e4",
            "eventName": "GetGroupPolicy",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2022-12-11 19:42:55",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": True,
            "recipientAccountId": "1231231234",
            "requestID": "f92dd1a7-ad07-4fef-9511-1081d2dd3585",
            "requestParameters": {"maxItems": 1000, "userName": "user-name"},
            "sourceIPAddress": "cloudformation.amazonaws.com",
            "userAgent": "cloudformation.amazonaws.com",
            "userIdentity": {
                "accessKeyId": "ASIAVKVYIOO7BDL4T5NG",
                "accountId": "1231231234",
                "arn": "arn:aws:sts::1231231234:assumed-role/AssumedRole-us-east-2/123123123456",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "AROAVKVYIOO7JN7TN7NSA:123123123456",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-12-11T19:42:54Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "1231231234",
                        "arn": "arn:aws:iam::1231231234:role/PAssumedRole-us-east-2",
                        "principalId": "AROAVKVYIOO7JN7TN7NSA",
                        "type": "Role",
                        "userName": "AssumedRole-us-east-2",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="List Attached Group Policies",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "883efb94-aa58-4512-beb7-10a5fffa33e4",
            "eventName": "ListAttachedGroupPolicies",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2022-12-11 19:42:55",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": True,
            "recipientAccountId": "1231231234",
            "requestID": "f92dd1a7-ad07-4fef-9511-1081d2dd3585",
            "requestParameters": {"maxItems": 1000, "userName": "user-name"},
            "sourceIPAddress": "cloudformation.amazonaws.com",
            "userAgent": "cloudformation.amazonaws.com",
            "userIdentity": {
                "accessKeyId": "ASIAVKVYIOO7BDL4T5NG",
                "accountId": "1231231234",
                "arn": "arn:aws:sts::1231231234:assumed-role/AssumedRole-us-east-2/123123123456",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "AROAVKVYIOO7JN7TN7NSA:123123123456",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-12-11T19:42:54Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "1231231234",
                        "arn": "arn:aws:iam::1231231234:role/PAssumedRole-us-east-2",
                        "principalId": "AROAVKVYIOO7JN7TN7NSA",
                        "type": "Role",
                        "userName": "AssumedRole-us-east-2",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="List Groups",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "883efb94-aa58-4512-beb7-10a5fffa33e4",
            "eventName": "ListGroups",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2022-12-11 19:42:55",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": True,
            "recipientAccountId": "1231231234",
            "requestID": "f92dd1a7-ad07-4fef-9511-1081d2dd3585",
            "requestParameters": {"maxItems": 1000, "userName": "user-name"},
            "sourceIPAddress": "cloudformation.amazonaws.com",
            "userAgent": "cloudformation.amazonaws.com",
            "userIdentity": {
                "accessKeyId": "ASIAVKVYIOO7BDL4T5NG",
                "accountId": "1231231234",
                "arn": "arn:aws:sts::1231231234:assumed-role/AssumedRole-us-east-2/123123123456",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "AROAVKVYIOO7JN7TN7NSA:123123123456",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-12-11T19:42:54Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "1231231234",
                        "arn": "arn:aws:iam::1231231234:role/PAssumedRole-us-east-2",
                        "principalId": "AROAVKVYIOO7JN7TN7NSA",
                        "type": "Role",
                        "userName": "AssumedRole-us-east-2",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="List Groups for User",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "883efb94-aa58-4512-beb7-10a5fffa33e4",
            "eventName": "ListGroupsForUser",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2022-12-11 19:42:55",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": True,
            "recipientAccountId": "1231231234",
            "requestID": "f92dd1a7-ad07-4fef-9511-1081d2dd3585",
            "requestParameters": {"maxItems": 1000, "userName": "user-name"},
            "sourceIPAddress": "cloudformation.amazonaws.com",
            "userAgent": "cloudformation.amazonaws.com",
            "userIdentity": {
                "accessKeyId": "ASIAVKVYIOO7BDL4T5NG",
                "accountId": "1231231234",
                "arn": "arn:aws:sts::1231231234:assumed-role/AssumedRole-us-east-2/123123123456",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "AROAVKVYIOO7JN7TN7NSA:123123123456",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-12-11T19:42:54Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "1231231234",
                        "arn": "arn:aws:iam::1231231234:role/PAssumedRole-us-east-2",
                        "principalId": "AROAVKVYIOO7JN7TN7NSA",
                        "type": "Role",
                        "userName": "AssumedRole-us-east-2",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Detach User Group",
        expected_result=False,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "883efb94-aa58-4512-beb7-10a5fffa33e4",
            "eventName": "DetachUserGroup",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2022-12-11 19:42:55",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": True,
            "recipientAccountId": "1231231234",
            "requestID": "f92dd1a7-ad07-4fef-9511-1081d2dd3585",
            "requestParameters": {"maxItems": 1000, "userName": "user-name"},
            "sourceIPAddress": "cloudformation.amazonaws.com",
            "userAgent": "cloudformation.amazonaws.com",
            "userIdentity": {
                "accessKeyId": "ASIAVKVYIOO7BDL4T5NG",
                "accountId": "1231231234",
                "arn": "arn:aws:sts::1231231234:assumed-role/AssumedRole-us-east-2/123123123456",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "AROAVKVYIOO7JN7TN7NSA:123123123456",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-12-11T19:42:54Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "1231231234",
                        "arn": "arn:aws:iam::1231231234:role/PAssumedRole-us-east-2",
                        "principalId": "AROAVKVYIOO7JN7TN7NSA",
                        "type": "Role",
                        "userName": "AssumedRole-us-east-2",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
]


@panther_managed
class AWSIAMGroupReadOnlyEvents(Rule):
    default_description = (
        "This rule captures multiple read/list events related to IAM group management in AWS Cloudtrail."
    )
    display_name = "AWS IAM Group Read Only Events"
    enabled = False
    default_reference = "https://attack.mitre.org/techniques/T1069/"
    default_runbook = "Examine other activities done by this user to determine whether or not activity is suspicious."
    default_severity = Severity.INFO
    tags = ["AWS", "Cloudtrail", "Configuration Required", "IAM", "MITRE"]
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.IAM.Group.Read.Only.Events-prototype"
    threshold = 2
    tests = awsiam_group_read_only_events_tests
    # arn allow list to suppress alerts
    ARN_ALLOW_LIST = []
    GROUP_ACTIONS = [
        "GetGroup",
        "GetGroupPolicy",
        "ListAttachedGroupPolicies",
        "ListGroupPolicies",
        "ListGroups",
        "ListGroupsForUser",
    ]

    def rule(self, event):
        event_arn = event.get("userIdentity", {}).get("arn", "<NO_ARN_FOUND>")
        # Return True if arn not in whitelist and event source is iam and event name is
        # present in read/list event_name list.
        if (
            event_arn not in self.ARN_ALLOW_LIST
            and event.get("eventSource", "<NO_EVENT_SOURCE_FOUND>") == "iam.amazonaws.com"
            and (event.get("eventName", "<NO_EVENT_NAME_FOUND>") in self.GROUP_ACTIONS)
        ):
            # continue on with analysis
            return True
        return False

    def title(self, event):
        return f"{event.get('userIdentity', {}).get('arn', '<NO_ARN_FOUND>')} IAM user group activity event found: {event.get('eventName', '<NO_EVENT_NAME_FOUND>')} in account {event.get('recipientAccountId', '<NO_RECIPIENT_ACCT_ID_FOUND>')} in region {event.get('awsRegion', '<NO_AWS_REGION_FOUND>')}."

    def dedup(self, event):
        # dedup via arn value
        return f"{event.get('userIdentity', {}).get('arn', '<NO_ARN_FOUND>')}"

    def alert_context(self, event):
        return aws_rule_context(event)



import re
from fnmatch import fnmatch

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import m365_alert_context

microsoft365_external_document_sharing_tests: list[RuleTest] = [
    RuleTest(
        name="Allowed Domain",
        expected_result=False,
        log={
            "AppAccessContext": {"AADSessionId": "aa-bb-cc", "CorrelationId": "dd-ee-ff"},
            "ClientIP": "1.2.3.4",
            "CreationTime": "2022-12-12 19:31:41",
            "EventData": "<Type>Edit</Type><MembersCanShareApplied>False</MembersCanShareApplied>",
            "EventSource": "SharePoint",
            "Id": "111-aa-234",
            "ItemType": "File",
            "ObjectId": "https://yourorg.sharepoint.com/personal/user_yourorg/Documents/importantsecrets.docx",
            "Operation": "AddedToSecureLink",
            "OrganizationId": "11-22-abc",
            "RecordType": 14,
            "Site": "aa-bb-dd-ee-ff",
            "SiteUrl": "https://yourorg.sharepoint.com/personal/user_yourorg",
            "SourceFileExtension": "docx",
            "SourceFileName": "importantsecrets.docx",
            "SourceRelativeUrl": "Documents/importantsecrets.docx",
            "TargetUserOrGroupName": "OUTSIDER@ALLOWEDDOMAIN.COM",
            "TargetUserOrGroupType": "Guest",
            "UserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            "UserId": "my.user@yourorg.onmicrosoft.com",
            "UserKey": "i:0h.f|membership|12345@live.com",
            "UserType": 0,
            "Workload": "OneDrive",
        },
    ),
    RuleTest(
        name="Allowed Folder",
        expected_result=False,
        log={
            "AppAccessContext": {"AADSessionId": "aa-bb-cc", "CorrelationId": "dd-ee-ff"},
            "ClientIP": "1.2.3.4",
            "CreationTime": "2022-12-12 19:31:41",
            "EventData": "<Type>Edit</Type><MembersCanShareApplied>False</MembersCanShareApplied>",
            "EventSource": "SharePoint",
            "Id": "111-aa-234",
            "ItemType": "File",
            "ObjectId": "https://yourorg.sharepoint.com/personal/user_yourorg/Documents/External/public.docx",
            "Operation": "AddedToSecureLink",
            "OrganizationId": "11-22-abc",
            "RecordType": 14,
            "Site": "aa-bb-dd-ee-ff",
            "SiteUrl": "https://yourorg.sharepoint.com/personal/user_yourorg",
            "SourceFileExtension": "docx",
            "SourceFileName": "public.docx",
            "SourceRelativeUrl": "Documents/External/public.docx",
            "TargetUserOrGroupName": "MARKETING@SAAS.COM",
            "TargetUserOrGroupType": "Guest",
            "UserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            "UserId": "my.user@yourorg.onmicrosoft.com",
            "UserKey": "i:0h.f|membership|12345@live.com",
            "UserType": 0,
            "Workload": "OneDrive",
        },
    ),
    RuleTest(
        name="Allowed User",
        expected_result=False,
        log={
            "AppAccessContext": {"AADSessionId": "aa-bb-cc", "CorrelationId": "dd-ee-ff"},
            "ClientIP": "1.2.3.4",
            "CreationTime": "2022-12-12 19:31:41",
            "EventData": "<Type>Edit</Type><MembersCanShareApplied>False</MembersCanShareApplied>",
            "EventSource": "SharePoint",
            "Id": "111-aa-234",
            "ItemType": "File",
            "ObjectId": "https://yourorg.sharepoint.com/personal/user_yourorg/Documents/importantsecrets.docx",
            "Operation": "AddedToSecureLink",
            "OrganizationId": "11-22-abc",
            "RecordType": 14,
            "Site": "aa-bb-dd-ee-ff",
            "SiteUrl": "https://yourorg.sharepoint.com/personal/user_yourorg",
            "SourceFileExtension": "docx",
            "SourceFileName": "importantsecrets.docx",
            "SourceRelativeUrl": "Documents/importantsecrets.docx",
            "TargetUserOrGroupName": "EXCEPTION@OUTSIDER.COM",
            "TargetUserOrGroupType": "Guest",
            "UserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            "UserId": "my.user@yourorg.onmicrosoft.com",
            "UserKey": "i:0h.f|membership|12345@live.com",
            "UserType": 0,
            "Workload": "OneDrive",
        },
    ),
    RuleTest(
        name="External user",
        expected_result=True,
        log={
            "AppAccessContext": {"AADSessionId": "aa-bb-cc", "CorrelationId": "dd-ee-ff"},
            "ClientIP": "1.2.3.4",
            "CreationTime": "2022-12-12 19:31:41",
            "EventData": "<Type>Edit</Type><MembersCanShareApplied>False</MembersCanShareApplied>",
            "EventSource": "SharePoint",
            "Id": "111-aa-234",
            "ItemType": "File",
            "ObjectId": "https://yourorg.sharepoint.com/personal/user_yourorg/Documents/importantsecrets.docx",
            "Operation": "AddedToSecureLink",
            "OrganizationId": "11-22-abc",
            "RecordType": 14,
            "Site": "aa-bb-dd-ee-ff",
            "SiteUrl": "https://yourorg.sharepoint.com/personal/user_yourorg",
            "SourceFileExtension": "docx",
            "SourceFileName": "importantsecrets.docx",
            "SourceRelativeUrl": "Documents/importantsecrets.docx",
            "TargetUserOrGroupName": "OUTSIDER@EXTERNAL.IO",
            "TargetUserOrGroupType": "Guest",
            "UserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            "UserId": "my.user@yourorg.onmicrosoft.com",
            "UserKey": "i:0h.f|membership|12345@live.com",
            "UserType": 0,
            "Workload": "OneDrive",
        },
    ),
    RuleTest(
        name="Internal Share",
        expected_result=False,
        log={
            "AppAccessContext": {"AADSessionId": "aa-bb-cc", "CorrelationId": "dd-ee-ff"},
            "ClientIP": "1.2.3.4",
            "CreationTime": "2022-12-12 19:31:41",
            "EventData": "<Type>Edit</Type><MembersCanShareApplied>False</MembersCanShareApplied>",
            "EventSource": "SharePoint",
            "Id": "111-aa-234",
            "ItemType": "File",
            "ObjectId": "https://yourorg.sharepoint.com/personal/user_yourorg/Documents/importantsecrets.docx",
            "Operation": "AddedToSecureLink",
            "OrganizationId": "11-22-abc",
            "RecordType": 14,
            "Site": "aa-bb-dd-ee-ff",
            "SiteUrl": "https://yourorg.sharepoint.com/personal/user_yourorg",
            "SourceFileExtension": "docx",
            "SourceFileName": "importantsecrets.docx",
            "SourceRelativeUrl": "Documents/importantsecrets.docx",
            "TargetUserOrGroupName": "ANOTHERUSER@MYCOMPANY.COM",
            "TargetUserOrGroupType": "NotGuest",
            "UserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            "UserId": "my.user@yourorg.onmicrosoft.com",
            "UserKey": "i:0h.f|membership|12345@live.com",
            "UserType": 0,
            "Workload": "OneDrive",
        },
    ),
]


@panther_managed
class Microsoft365ExternalDocumentSharing(Rule):
    default_description = "Document shared externally"
    display_name = "Microsoft365 External Document Sharing"
    reports = {"MITRE ATT&CK": ["TA0009:T1039"]}
    default_runbook = "Check the document metadata to ensure it is not a sensitive document."
    default_reference = "https://support.microsoft.com/en-us/topic/manage-sharing-with-external-users-in-microsoft-365-small-business-2951a85f-c970-4375-aa4f-6b0d7035fe35#:~:text=Top%20of%20Page-,Turn%20external%20sharing%20on%20or%20off,-The%20ability%20to"
    default_severity = Severity.LOW
    log_types = [LogType.MICROSOFT365_AUDIT_SHAREPOINT]
    id = "Microsoft365.External.Document.Sharing-prototype"
    tests = microsoft365_external_document_sharing_tests
    email_regex = re.compile("([A-Za-z0-9]+[.-_])*[A-Za-z0-9]+@[A-Za-z0-9-]+(\\.[A-Z|a-z]{2,})+")
    ALLOWED_DOMAINS = ["mycompany.com", "alloweddomain.com"]  # should be in lowercase
    ALLOWED_USERS = ["exception@outsider.com"]  # should be in lowercase
    ALLOWED_PATHS = ["*/External/*", "External/*"]

    def allowed_path(self, relative_url):
        for path in self.ALLOWED_PATHS:
            if fnmatch(relative_url, path):
                return True
        return False

    def rule(self, event):
        if event.get("Operation", "") == "AnonymousLinkCreated":
            return not self.allowed_path(event.get("SourceRelativeUrl"))
        if event.get("Operation", "") == "AddedToSecureLink":
            if self.allowed_path(event.get("SourceRelativeUrl")):
                return False
            target = event.get("TargetUserOrGroupName", "")
            if target.lower() in self.ALLOWED_USERS:
                return False
            if re.fullmatch(self.email_regex, target):
                if target.split("@")[1].lower() not in self.ALLOWED_DOMAINS:
                    return True
        return False

    def title(self, event):
        return f"Microsoft365: [{event.get('SourceRelativeUrl')}] has been shared with external users by [{event.get('UserId', '<user-not-found>')}]"

    def alert_context(self, event):
        return m365_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

gcp_unused_regions_tests: list[RuleTest] = [
    RuleTest(
        name="GCE Instance Terminated",
        expected_result=False,
        log={
            "logName": "projects/western-verve-225918/logs/cloudaudit.googleapis.com%2Factivity",
            "severity": "NOTICE",
            "insertId": "81xwjyd5vh0",
            "resource": {
                "type": "gce_instance",
                "labels": {
                    "instance_id": "5423650206360236277",
                    "project_id": "western-verve-225918",
                    "zone": "europe-north1-a",
                },
            },
            "timestamp": "2020-05-15 17:55:19.303000000",
            "receiveTimestamp": "2020-05-15 17:55:20.210068584",
            "operation": {
                "id": "operation-1589565287793-5a5b382538481-3da9816c-ce929997",
                "producer": "compute.googleapis.com",
                "last": True,
            },
            "protoPayload": {
                "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "serviceName": "compute.googleapis.com",
                "methodName": "v1.compute.instances.delete",
                "resourceName": "projects/western-verve-225918/zones/europe-north1-a/instances/instance-3",
                "authenticationInfo": {"principalEmail": "jack.naglieri@runpanther.io"},
                "requestMetadata": {
                    "callerIP": "136.24.229.58",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36,gzip(gfe),gzip(gfe)",
                },
                "request": {"@type": "type.googleapis.com/compute.instances.delete"},
            },
        },
    ),
    RuleTest(
        name="GCE Create Instance in SouthAmerica",
        expected_result=True,
        log={
            "protoPayload": {
                "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user.name@runpanther.io"},
                "requestMetadata": {
                    "callerIp": "136.24.229.58",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36,gzip(gfe)",
                },
                "serviceName": "compute.googleapis.com",
                "methodName": "beta.compute.instances.insert",
                "resourceName": "projects/western-verve-123456/zones/asia-northeast1-b/instances/instance-5",
                "request": {"@type": "type.googleapis.com/compute.instances.insert"},
            },
            "insertId": "-5tqx5fd4mj8",
            "resource": {
                "type": "gce_instance",
                "labels": {
                    "instance_id": "8498166540490993880",
                    "project_id": "western-verve-123456",
                    "zone": "southamerica-east1-b",
                },
            },
            "timestamp": "2020-05-15T17:15:42.415Z",
            "severity": "NOTICE",
            "logName": "projects/western-verve-123456/logs/cloudaudit.googleapis.com%2Factivity",
            "operation": {
                "id": "operation-1589562934964-5a5b2f61631d6-cc67597a-98092474",
                "producer": "compute.googleapis.com",
                "last": True,
            },
            "receiveTimestamp": "2020-05-15T17:15:43.377082868Z",
        },
    ),
    RuleTest(
        name="Create GCS in Asia",
        expected_result=True,
        log={
            "protoPayload": {
                "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "status": {},
                "authenticationInfo": {"principalEmail": "user.name@runpanther.io"},
                "requestMetadata": {
                    "callerIp": "136.24.229.58",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36,gzip(gfe),gzip(gfe)",
                    "requestAttributes": {"time": "2020-05-15T17:25:07.810848781Z", "auth": {}},
                    "destinationAttributes": {},
                },
                "serviceName": "storage.googleapis.com",
                "methodName": "storage.buckets.create",
                "authorizationInfo": [
                    {
                        "resource": "projects/_/buckets/jacks-test-bucket-200",
                        "permission": "storage.buckets.create",
                        "granted": True,
                        "resourceAttributes": {},
                    },
                ],
                "resourceName": "projects/_/buckets/jacks-test-bucket-200",
                "serviceData": {
                    "@type": "type.googleapis.com/google.iam.v1.logging.AuditData",
                    "policyDelta": {
                        "bindingDeltas": [
                            {
                                "action": "ADD",
                                "role": "roles/storage.legacyBucketOwner",
                                "member": "projectEditor:western-verve-123456",
                            },
                            {
                                "action": "ADD",
                                "role": "roles/storage.legacyBucketOwner",
                                "member": "projectOwner:western-verve-123456",
                            },
                            {
                                "action": "ADD",
                                "role": "roles/storage.legacyBucketReader",
                                "member": "projectViewer:western-verve-123456",
                            },
                        ],
                    },
                },
                "request": {
                    "defaultObjectAcl": {
                        "bindings": [
                            {
                                "role": "roles/storage.legacyObjectReader",
                                "members": ["projectViewer:western-verve-123456"],
                            },
                            {
                                "members": ["projectOwner:western-verve-123456", "projectEditor:western-verve-123456"],
                                "role": "roles/storage.legacyObjectOwner",
                            },
                        ],
                        "@type": "type.googleapis.com/google.iam.v1.Policy",
                    },
                },
                "resourceLocation": {"currentLocations": ["asia-northeast2"]},
            },
            "insertId": "c7rgc9c178",
            "resource": {
                "type": "gcs_bucket",
                "labels": {
                    "bucket_name": "jacks-test-bucket-200",
                    "project_id": "western-verve-123456",
                    "location": "asia-northeast2",
                },
            },
            "timestamp": "2020-05-15T17:25:07.807169539Z",
            "severity": "NOTICE",
            "logName": "projects/western-verve-123456/logs/cloudaudit.googleapis.com%2Factivity",
            "receiveTimestamp": "2020-05-15T17:25:09.393448555Z",
        },
    ),
    RuleTest(
        name="BigQuery access log (does not have standard attribute: resource.labels.location)",
        expected_result=False,
        log={
            "insertId": "v3a96bedw1us",
            "logName": "projects/western-verve-123456/logs/cloudaudit.googleapis.com%2Fdata_access",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user.name@runpanther.io"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "bigquery.jobs.create",
                        "resource": "projects/western-verve-123456",
                        "resourceAttributes": {},
                    },
                ],
                "methodName": "jobservice.insert",
                "requestMetadata": {
                    "callerIP": "192.0.2.0",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:92.0) Gecko/20100101 Firefox/92.0,gzip(gfe),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {},
                },
                "resourceName": "projects/western-verve-123456/jobs",
                "serviceData": {
                    "@type": "type.googleapis.com/google.cloud.bigquery.logging.v1.AuditData",
                    "jobInsertRequest": {
                        "resource": {
                            "jobConfiguration": {
                                "dryRun": True,
                                "query": {
                                    "createDisposition": "CREATE_IF_NEEDED",
                                    "defaultDataset": {},
                                    "destinationTable": {},
                                    "query": "select * from no_such_table",
                                    "queryPriority": "QUERY_INTERACTIVE",
                                    "writeDisposition": "WRITE_EMPTY",
                                },
                            },
                            "jobName": {"location": "US", "projectId": "western-verve-123456"},
                        },
                    },
                    "jobInsertResponse": {
                        "resource": {
                            "jobConfiguration": {},
                            "jobName": {},
                            "jobStatistics": {},
                            "jobStatus": {"error": {}, "state": "PENDING"},
                        },
                    },
                },
                "serviceName": "bigquery.googleapis.com",
                "status": {"code": 11, "message": "Syntax error: Unexpected end of script at [17:7]"},
            },
            "receiveTimestamp": "2021-10-19 16:09:26.351861539",
            "resource": {"labels": {"project_id": "western-verve-123456"}, "type": "bigquery_resource"},
            "severity": "ERROR",
            "timestamp": "2021-10-19 16:09:26.013200000",
        },
    ),
]


@panther_managed
class GCPUnusedRegions(Rule):
    id = "GCP.UnusedRegions-prototype"
    display_name = "GCP Resource in Unused Region"
    enabled = False
    dedup_period_minutes = 15
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["GCP", "Database", "Configuration Required", "Defense Evasion:Unused/Unsupported Cloud Regions"]
    reports = {"MITRE ATT&CK": ["TA0005:T1535"]}
    default_severity = Severity.MEDIUM
    default_description = (
        "Adversaries may create cloud instances in unused geographic service regions in order to evade detection.\n"
    )
    default_runbook = "Validate the user making the request and the resource created."
    default_reference = "https://cloud.google.com/docs/geography-and-regions"
    summary_attributes = ["severity", "p_any_ip_addresses", "p_any_domain_names"]
    tests = gcp_unused_regions_tests
    # 'asia',
    # 'australia',
    # 'eu',
    # 'northamerica',
    # 'southamerica',
    APPROVED_ACTIVE_REGIONS = {"us"}

    def _resource_in_active_region(self, location):
        # return False if location is None, meaning the event did not have a location attribute
        # in any of the places we would expect to find one.
        if location is False:
            return False
        return not any(location.startswith(active_region) for active_region in self.APPROVED_ACTIVE_REGIONS)

    def _get_location_or_zone(self, event):
        resource = event.get("resource")
        if not resource:
            return False
        resource_location = deep_get(resource, "labels", "location")
        if resource_location:
            return resource_location
        resource_zone = deep_get(resource, "labels", "zone")
        if resource_zone:
            return resource_zone
        return False

    def rule(self, event):
        method_name = deep_get(event, "protoPayload", "methodName", default="<UNKNOWN_METHOD>")
        if not method_name.endswith(("insert", "create")):
            return False
        return self._resource_in_active_region(self._get_location_or_zone(event))

    def title(self, event):
        return f"GCP resource(s) created in unused region/zone in project {deep_get(event, 'resource', 'labels', 'project_id', default='<UNKNOWN_PROJECT>')}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get

gcpk8s_pod_using_host_pid_namespace_tests: list[RuleTest] = [
    RuleTest(
        name="triggers",
        expected_result=True,
        log={
            "authorizationInfo": [
                {
                    "granted": True,
                    "permission": "io.k8s.core.v1.pods.create",
                    "resource": "core/v1/namespaces/default/pods/nginx-test",
                },
            ],
            "protoPayload": {"methodName": "io.k8s.core.v1.pods.create", "request": {"spec": {"hostPID": True}}},
        },
    ),
    RuleTest(
        name="ignore",
        expected_result=False,
        log={
            "authorizationInfo": [
                {
                    "granted": True,
                    "permission": "io.k8s.core.v1.pods.create",
                    "resource": "core/v1/namespaces/default/pods/nginx-test",
                },
            ],
            "protoPayload": {"methodName": "io.k8s.core.v1.pods.create", "request": {"spec": {"hostPID": False}}},
        },
    ),
]


@panther_managed
class GCPK8sPodUsingHostPIDNamespace(Rule):
    id = "GCP.K8s.Pod.Using.Host.PID.Namespace-prototype"
    display_name = "GCP K8s Pod Using Host PID Namespace"
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["GCP", "Optional"]
    default_severity = Severity.MEDIUM
    default_description = "This detection monitors for any pod creation or modification using the host PID namespace. The Host PID namespace enables a pod and its containers to have direct access and share the same view as of the host’s processes. This can offer a powerful escape hatch to the underlying host."
    default_runbook = "Investigate a reason of creating a pod using the host PID namespace. Advise that it is discouraged practice. Create ticket if appropriate."
    reports = {"MITRE ATT&CK": ["TA0004:T1611", "TA0002:T1610"]}
    default_reference = "https://medium.com/snowflake/from-logs-to-detection-using-snowflake-and-panther-to-detect-k8s-threats-d72f70a504d7"
    tests = gcpk8s_pod_using_host_pid_namespace_tests
    METHODS_TO_CHECK = ["io.k8s.core.v1.pods.create", "io.k8s.core.v1.pods.update", "io.k8s.core.v1.pods.patch"]

    def rule(self, event):
        method = deep_get(event, "protoPayload", "methodName")
        request_host_pid = deep_get(event, "protoPayload", "request", "spec", "hostPID")
        response_host_pid = deep_get(event, "protoPayload", "responce", "spec", "hostPID")
        if (request_host_pid is True or response_host_pid is True) and method in self.METHODS_TO_CHECK:
            return True
        return False

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        project_id = deep_get(event, "resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: [{actor}] created or modified pod using the host PID namespace in project [{project_id}]"

    def alert_context(self, event):
        return gcp_alert_context(event)



import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import DataModel, DataModelMapping, LogType
from pypanther.helpers.panther_azuresignin_helpers import actor_user, is_sign_in_event
from pypanther.helpers.panther_base_helpers import deep_get


def get_event_type(event):
    if not is_sign_in_event(event):
        return None

    error_code = deep_get(event, "properties", "status", "errorCode", default=0)
    if error_code == 0:
        return event_type.SUCCESSFUL_LOGIN
    return event_type.FAILED_LOGIN


def get_actor_user(event):
    return actor_user(event)


class StandardAzureAuditSignIn(DataModel):
    id: str = "Standard.Azure.Audit.SignIn"
    display_name: str = "Azure SignIn Logs DataModel"
    enabled: bool = True
    log_types: list[str] = [LogType.AZURE_AUDIT]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="actor_user", method=get_actor_user),
        DataModelMapping(name="event_type", method=get_event_type),
        DataModelMapping(name="source_ip", path="$.properties.ipAddress"),
    ]



"""Used to define global event_types"""

# a place to hold event type constants used among many data models, rules, or policies
ADMIN_ROLE_ASSIGNED = "admin_role_assigned"
FAILED_LOGIN = "failed_login"
MFA_DISABLED = "mfa_disabled"
MFA_ENABLED = "mfa_enabled"
MFA_RESET = "mfa_reset"
SUCCESSFUL_LOGIN = "successful_login"
SUCCESSFUL_LOGOUT = "successful_logout"
USER_ACCOUNT_CREATED = "user_account_created"
# ACCOUNT_CREATED refers to an account not associated with a specific user,
# such as a billing account
ACCOUNT_CREATED = "account_created"
# USER_ACCOUNT_CREATED refers to an account that is associated with one user or service user
USER_ACCOUNT_DELETED = "user_account_deleted"
USER_ACCOUNT_MODIFIED = "user_account_modified"
USER_GROUP_CREATED = "user_group_created"
USER_GROUP_MODIFIED = "user_group_modified"
USER_GROUP_DELETED = "user_group_deleted"
USER_ROLE_CREATED = "user_role_created"
USER_ROLE_MODIFIED = "user_role_modified"
USER_ROLE_DELETED = "user_role_deleted"
# ADMIN_MFA_DISABLED refers to MFA being disabled for an entire org, account, or similar
ADMIN_MFA_DISABLED = "admin_mfa_disabled"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

osquery_mac_application_firewall_settings_tests: list[RuleTest] = [
    RuleTest(
        name="ALF Disabled",
        expected_result=True,
        log={
            "name": "pack_incident-response_alf",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "logging_enabled": "0",
                "stealth_enabled": "0",
                "firewall_unload": "0",
                "allow_signed_enabled": "0",
                "global_state": "0",
                "logging_option": "0",
                "version": "1.6",
            },
        },
    ),
    RuleTest(
        name="ALF Enabled",
        expected_result=False,
        log={
            "name": "pack_incident-response_alf",
            "action": "added",
            "hostIdentifier": "test-host",
            "columns": {
                "logging_enabled": "1",
                "stealth_enabled": "1",
                "firewall_unload": "0",
                "allow_signed_enabled": "1",
                "global_state": "1",
                "logging_option": "0",
                "version": "1.6",
            },
        },
    ),
]


@panther_managed
class OsqueryMacApplicationFirewallSettings(Rule):
    id = "Osquery.Mac.ApplicationFirewallSettings-prototype"
    display_name = "MacOS ALF is misconfigured"
    log_types = [LogType.OSQUERY_DIFFERENTIAL]
    tags = ["Osquery", "MacOS", "Security Control", "Defense Evasion:Impair Defenses"]
    reports = {"CIS": ["2.6.3", "2.6.4"], "MITRE ATT&CK": ["TA0005:T1562"]}
    default_severity = Severity.HIGH
    default_description = "The application level firewall blocks unwanted network connections made to your computer from other computers on your network.\n"
    default_runbook = "Re-enable the firewall manually or with configuration management"
    default_reference = "https://support.apple.com/en-us/HT201642"
    summary_attributes = ["name", "hostIdentifier", "action"]
    tests = osquery_mac_application_firewall_settings_tests
    QUERIES = {"pack_incident-response_alf", "pack/mac-cis/ApplicationFirewall"}

    def rule(self, event):
        if event.get("name") not in self.QUERIES:
            return False
        if event.get("action") != "added":
            return False
        # 0 If the firewall is disabled
        # 1 If the firewall is enabled with exceptions
        # 2 If the firewall is configured to block all incoming connections
        # Stealth mode is a best practice to avoid responding to unsolicited probes
        return (
            int(deep_get(event, "columns", "global_state")) == 0
            or int(deep_get(event, "columns", "stealth_enabled")) == 0
        )

    def title(self, event):
        return f"MacOS firewall disabled on [{event.get('hostIdentifier')}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

atlassian_user_logged_in_as_user_tests: list[RuleTest] = [
    RuleTest(
        name="Admin impersonated user successfully",
        expected_result=True,
        log={
            "attributes": {
                "action": "user_logged_in_as_user",
                "actor": {
                    "email": "example.admin@example.com",
                    "id": "1234567890abcdefghijklmn",
                    "name": "Example Admin",
                },
                "container": [
                    {
                        "attributes": {"siteHostName": "https://example.atlassian.net", "siteName": "example"},
                        "id": "12345678-abcd-9012-efgh-1234567890abcd",
                        "links": {"alt": "https://example.atlassian.net"},
                        "type": "sites",
                    },
                ],
                "context": [
                    {
                        "attributes": {
                            "accountType": "atlassian",
                            "email": "example.user@example.io",
                            "name": "example.user@example.io",
                        },
                        "type": "users",
                    },
                ],
                "time": "2022-12-15T00:35:15.890Z",
            },
            "id": "2508d209-3336-4763-89a0-aceaf1322fcf",
            "message": {"content": "Logged in as example.user@example.io", "format": "simple"},
        },
    ),
    RuleTest(
        name="user_logged_in_as_user not in log",
        expected_result=False,
        log={
            "attributes": {
                "action": "user_login",
                "actor": {
                    "email": "example.admin@example.com",
                    "id": "1234567890abcdefghijklmn",
                    "name": "Example Admin",
                },
                "container": [
                    {
                        "attributes": {"siteHostName": "https://example.atlassian.net", "siteName": "example"},
                        "id": "12345678-abcd-9012-efgh-1234567890abcd",
                        "links": {"alt": "https://example.atlassian.net"},
                        "type": "sites",
                    },
                ],
                "context": [
                    {
                        "attributes": {
                            "accountType": "atlassian",
                            "email": "example.user@example.io",
                            "name": "example.user@example.io",
                        },
                        "type": "users",
                    },
                ],
                "time": "2022-12-15T00:35:15.890Z",
            },
            "id": "2508d209-3336-4763-89a0-aceaf1322fcf",
            "message": {"content": "Logged in as example.user@example.io", "format": "simple"},
        },
    ),
]


@panther_managed
class AtlassianUserLoggedInAsUser(Rule):
    display_name = "Atlassian admin impersonated another user"
    id = "Atlassian.User.LoggedInAsUser-prototype"
    default_severity = Severity.HIGH
    log_types = [LogType.ATLASSIAN_AUDIT]
    tags = ["Atlassian", "User impersonation"]
    default_description = "Reports when an Atlassian user logs in (impersonates) another user.\n"
    default_runbook = "Validate that the Atlassian admin did log in (impersonate) as another user.\n"
    default_reference = "https://support.atlassian.com/user-management/docs/log-in-as-another-user/"
    tests = atlassian_user_logged_in_as_user_tests

    def rule(self, event):
        return deep_get(event, "attributes", "action", default="<unknown-action>") == "user_logged_in_as_user"

    def title(self, event):
        actor = deep_get(event, "attributes", "actor", "email", default="<unknown-email>")
        context = deep_get(event, "attributes", "context", default=[{}])
        impersonated_user = context[0].get("attributes", {}).get("email", "<unknown-email>")
        return f"{actor} logged in as {impersonated_user}."

    def alert_context(self, event):
        return {
            "Timestamp": deep_get(event, "attributes", "time", default="<unknown-time>"),
            "Actor": deep_get(event, "attributes", "actor", "email", default="<unknown-actor-email>"),
            "Impersonated user": deep_get(event, "attributes", "context", default=[{}])[0]
            .get("attributes", {})
            .get("email", "<unknown-email>"),
            "Event ID": event.get("id"),
        }



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

teleport_suspicious_commands_tests: list[RuleTest] = [
    RuleTest(
        name="Echo command",
        expected_result=False,
        log={
            "argv": [],
            "cgroup_id": 4294967537,
            "code": "T4000I",
            "ei": 15,
            "event": "session.command",
            "login": "root",
            "namespace": "default",
            "path": "/bin/echo",
            "pid": 7143,
            "ppid": 7115,
            "program": "echo",
            "return_code": 0,
            "server_id": "e75992b4-9e27-456f-b1c9-7a32da83c661",
            "sid": "8a3fc038-785b-43f3-8737-827b3e25fe5b",
            "time": "2020-08-17T17:40:37.491Z",
            "uid": "8eaf8f39-09d4-4a42-a22a-65163d2af702",
            "user": "panther",
        },
    ),
    RuleTest(
        name="Netcat command",
        expected_result=True,
        log={
            "argv": ["-l", "-p", "11434"],
            "cgroup_id": 4294967537,
            "code": "T4000I",
            "ei": 15,
            "event": "session.command",
            "login": "root",
            "namespace": "default",
            "path": "/bin/nc",
            "pid": 7143,
            "ppid": 7115,
            "program": "nc",
            "return_code": 0,
            "server_id": "e75992b4-9e27-456f-b1c9-7a32da83c661",
            "sid": "8a3fc038-785b-43f3-8737-827b3e25fe5b",
            "time": "2020-08-17T17:40:37.491Z",
            "uid": "8eaf8f39-09d4-4a42-a22a-65163d2af702",
            "user": "panther",
        },
    ),
]


@panther_managed
class TeleportSuspiciousCommands(Rule):
    id = "Teleport.SuspiciousCommands-prototype"
    display_name = "Teleport Suspicious Commands Executed"
    log_types = [LogType.GRAVITATIONAL_TELEPORT_AUDIT]
    tags = ["SSH", "Execution:Command and Scripting Interpreter"]
    default_severity = Severity.MEDIUM
    default_description = "A user has invoked a suspicious command that could lead to a host compromise"
    reports = {"MITRE ATT&CK": ["TA0002:T1059"]}
    default_reference = "https://goteleport.com/docs/management/admin/"
    default_runbook = "Find related commands within the time window and determine if the command was invoked legitimately. Examine the arguments to determine how the command was used and reach out to the user to verify the intentions.\n"
    summary_attributes = ["event", "code", "user", "program", "path", "return_code", "login", "server_id", "sid"]
    tests = teleport_suspicious_commands_tests
    SUSPICIOUS_COMMANDS = {"nc", "wget"}

    def rule(self, event):
        if event.get("event") != "session.command":
            return False
        # Ignore commands without arguments
        if not event.get("argv"):
            return False
        return event.get("program") in self.SUSPICIOUS_COMMANDS

    def title(self, event):
        return f"User [{event.get('user', '<UNKNOWN_USER>')}] has executed the command [{event.get('program', '<UNKNOWN_PROGRAM>')}] on [{event.get('cluster_name', '<UNKNOWN_CLUSTER>')}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

github_repo_visibility_change_tests: list[RuleTest] = [
    RuleTest(
        name="GitHub - Repo Visibility Change",
        expected_result=True,
        log={
            "actor": "cat",
            "action": "repo.access",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
        },
    ),
    RuleTest(
        name="GitHub - Repo disabled",
        expected_result=False,
        log={
            "actor": "cat",
            "action": "repo.disable",
            "created_at": 1621305118553,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
        },
    ),
]


@panther_managed
class GithubRepoVisibilityChange(Rule):
    id = "Github.Repo.VisibilityChange-prototype"
    display_name = "GitHub Repository Visibility Change"
    log_types = [LogType.GITHUB_AUDIT]
    tags = ["GitHub", "Exfiltration:Exfiltration Over Web Service"]
    reports = {"MITRE ATT&CK": ["TA0010:T1567"]}
    default_reference = "https://docs.github.com/en/repositories/managing-your-repositorys-settings-and-features/managing-repository-settings/setting-repository-visibility"
    default_severity = Severity.HIGH
    default_description = "Detects when an organization repository visibility changes."
    tests = github_repo_visibility_change_tests

    def rule(self, event):
        return event.get("action") == "repo.access"

    def title(self, event):
        repo_access_link = f"https://github.com/{event.get('repo', '<UNKNOWN_REPO>')}/settings/access"
        return f"Repository [{event.get('repo', '<UNKNOWN_REPO>')}] visibility changed. View current visibility here: {repo_access_link}"



import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import DataModel, DataModelMapping, LogType


def get_event_type(event):
    # currently, only tracking a handful of event types
    if event.get("event_type_id") == 72 and event.get("privilege_name") == "Super user":
        return event_type.ADMIN_ROLE_ASSIGNED
    if event.get("event_type_id") == 6:
        return event_type.FAILED_LOGIN
    if event.get("event_type_id") == 5:
        return event_type.SUCCESSFUL_LOGIN
    if event.get("event_type_id") == 13:
        return event_type.USER_ACCOUNT_CREATED
    return None


class StandardOneLoginEvents(DataModel):
    id: str = "Standard.OneLogin.Events"
    display_name: str = "OneLogin Events"
    enabled: bool = True
    log_types: list[str] = [LogType.ONELOGIN_EVENTS]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="actor_user", path="actor_user_name"),
        DataModelMapping(name="assigned_admin_role", path="privilege_name"),
        DataModelMapping(name="event_type", method=get_event_type),
        DataModelMapping(name="source_ip", path="ipaddr"),
        DataModelMapping(name="user", path="user_name"),
        DataModelMapping(name="user_account_id", path="user_id"),
        DataModelMapping(name="user_agent", path="user_agent"),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import github_alert_context

github_organization_app_integration_installed_tests: list[RuleTest] = [
    RuleTest(
        name="App Integration Installation",
        expected_result=True,
        log={
            "_document_id": "A-2345",
            "action": "integration_installation.create",
            "actor": "user_name",
            "actor_location": {"country_code": "US"},
            "at_sign_timestamp": "2022-12-11 05:28:05.542",
            "created_at": "2022-12-11 05:28:05.542",
            "name": "Microsoft Teams for GitHub",
            "org": "your-organization",
            "p_any_usernames": ["user_name"],
        },
    ),
    RuleTest(
        name="App Integration Installation-2",
        expected_result=True,
        log={
            "_document_id": "A-1234",
            "action": "integration_installation.create",
            "actor": "leetboy",
            "actor_location": {"country_code": "US"},
            "at_sign_timestamp": "2022-12-02 17:40:08.671",
            "created_at": "2022-12-02 17:40:08.671",
            "name": "Datadog CI",
            "org": "example-io",
        },
    ),
    RuleTest(
        name="Repository Archived",
        expected_result=False,
        log={
            "action": "repo.archived",
            "actor": "cat",
            "created_at": 1621305118553.0,
            "org": "my-org",
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
        },
    ),
]


@panther_managed
class GithubOrganizationAppIntegrationInstalled(Rule):
    default_description = "An application integration was installed to your organization's Github account by someone in your organization."
    display_name = "Github Organization App Integration Installed"
    default_reference = (
        "https://docs.github.com/en/enterprise-server@3.4/developers/apps/managing-github-apps/installing-github-apps"
    )
    default_runbook = "Confirm that the app integration installation was a desired behavior."
    default_severity = Severity.LOW
    tags = ["Application Installation", "Github"]
    log_types = [LogType.GITHUB_AUDIT]
    id = "Github.Organization.App.Integration.Installed-prototype"
    summary_attributes = ["actor", "name"]
    tests = github_organization_app_integration_installed_tests
    # def dedup(event):
    #  (Optional) Return a string which will be used to deduplicate similar alerts.
    # return ''

    def rule(self, event):
        # Return True to match the log event and trigger an alert.
        # Creates a new alert if the event's action was ""
        return event.get("action") == "integration_installation.create"

    def title(self, event):
        # (Optional) Return a string which will be shown as the alert title.
        # If no 'dedup' function is defined, the return value of this method
        # will act as deduplication string.
        return f" Github User [{event.get('actor', {})}] in [{event.get('org')}] installed the following integration: [{event.get('name')}]."

    def alert_context(self, event):
        #  (Optional) Return a dictionary with additional data to be included in the
        #  alert sent to the SNS/SQS/Webhook destination
        return github_alert_context(event)



import json

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import m365_alert_context

microsoft365_mfa_disabled_tests: list[RuleTest] = [
    RuleTest(
        name="MFA Add Event",
        expected_result=False,
        log={
            "Actor": [
                {"ID": "Azure MFA StrongAuthenticationService", "Type": 1},
                {"ID": "ABC-123", "Type": 2},
                {"ID": "ServicePrincipal_123-abc", "Type": 2},
                {"ID": "321-cba", "Type": 2},
                {"ID": "ServicePrincipal", "Type": 2},
            ],
            "ActorContextId": "123-abc-456",
            "AzureActiveDirectoryEventType": 1,
            "CreationTime": "2022-12-12 17:28:35",
            "ExtendedProperties": [
                {"Name": "additionalDetails", "Value": '{"UserType":"Member"}'},
                {"Name": "extendedAuditEventCategory", "Value": "User"},
            ],
            "Id": "123-abc-123",
            "InterSystemsId": "abc-123-321",
            "IntraSystemId": "aa-bbb-333",
            "ModifiedProperties": [
                {
                    "Name": "StrongAuthenticationMethod",
                    "NewValue": '[{"Default": true,"MethodType": 7}]',
                    "OldValue": "[]",
                },
                {"Name": "Included Updated Properties", "NewValue": "StrongAuthenticationMethod", "OldValue": ""},
                {"Name": "TargetId.UserType", "NewValue": "Member", "OldValue": ""},
            ],
            "ObjectId": "sample.user@yourorg.onmicrosoft.com",
            "Operation": "Update user.",
            "OrganizationId": "111-222-333",
            "RecordType": 8,
            "ResultStatus": "Success",
            "SupportTicketId": "",
            "Target": [
                {"ID": "User_111-222-bbb", "Type": 2},
                {"ID": "111-aa-bbb-321", "Type": 2},
                {"ID": "User", "Type": 2},
                {"ID": "sample.user@yourorg.onmicrosoft.com", "Type": 5},
                {"ID": "123abcdef", "Type": 3},
            ],
            "TargetContextId": "aaa-bb-222",
            "UserId": "ServicePrincipal_aa-bb-ccc",
            "UserKey": "Not Available",
            "UserType": 4,
            "Workload": "AzureActiveDirectory",
        },
    ),
    RuleTest(
        name="MFA Remove event",
        expected_result=True,
        log={
            "Actor": [
                {"ID": "Azure MFA StrongAuthenticationService", "Type": 1},
                {"ID": "ABC-123", "Type": 2},
                {"ID": "ServicePrincipal_123-abc", "Type": 2},
                {"ID": "321-cba", "Type": 2},
                {"ID": "ServicePrincipal", "Type": 2},
            ],
            "ActorContextId": "123-abc-456",
            "AzureActiveDirectoryEventType": 1,
            "CreationTime": "2022-12-12 17:28:35",
            "ExtendedProperties": [
                {"Name": "additionalDetails", "Value": '{"UserType":"Member"}'},
                {"Name": "extendedAuditEventCategory", "Value": "User"},
            ],
            "Id": "123-abc-123",
            "InterSystemsId": "abc-123-321",
            "IntraSystemId": "aa-bbb-333",
            "ModifiedProperties": [
                {
                    "Name": "StrongAuthenticationMethod",
                    "NewValue": "[]",
                    "OldValue": '[{"Default": true,"MethodType": 7}]',
                },
                {"Name": "Included Updated Properties", "NewValue": "StrongAuthenticationMethod", "OldValue": ""},
                {"Name": "TargetId.UserType", "NewValue": "Member", "OldValue": ""},
            ],
            "ObjectId": "sample.user@yourorg.onmicrosoft.com",
            "Operation": "Update user.",
            "OrganizationId": "111-222-333",
            "RecordType": 8,
            "ResultStatus": "Success",
            "SupportTicketId": "",
            "Target": [
                {"ID": "User_111-222-bbb", "Type": 2},
                {"ID": "111-aa-bbb-321", "Type": 2},
                {"ID": "User", "Type": 2},
                {"ID": "sample.user@yourorg.onmicrosoft.com", "Type": 5},
                {"ID": "123abcdef", "Type": 3},
            ],
            "TargetContextId": "aaa-bb-222",
            "UserId": "ServicePrincipal_aa-bb-ccc",
            "UserKey": "Not Available",
            "UserType": 4,
            "Workload": "AzureActiveDirectory",
        },
    ),
]


@panther_managed
class Microsoft365MFADisabled(Rule):
    default_description = "A user's MFA has been removed"
    display_name = "Microsoft365 MFA Disabled"
    reports = {"MITRE ATT&CK": ["TA003:T1556", "TA005:T1556", "TA006:T1556"]}
    default_runbook = "Depending on company policy, either suggest or require the user re-enable two step verification."
    default_reference = "https://learn.microsoft.com/en-us/microsoft-365/admin/security-and-compliance/set-up-multi-factor-authentication?view=o365-worldwide"
    default_severity = Severity.LOW
    log_types = [LogType.MICROSOFT365_AUDIT_AZURE_ACTIVE_DIRECTORY]
    id = "Microsoft365.MFA.Disabled-prototype"
    tests = microsoft365_mfa_disabled_tests

    def rule(self, event):
        if event.get("Operation", "") == "Update user.":
            modified_properties = event.get("ModifiedProperties", [])
            for prop in modified_properties:
                if prop.get("Name", "") == "StrongAuthenticationMethod":
                    new_value = prop.get("NewValue")
                    old_value = prop.get("OldValue")
                    if isinstance(new_value, str):
                        new_value = json.loads(new_value)
                    if isinstance(old_value, str):
                        old_value = json.loads(old_value)
                    if old_value and (not new_value):
                        return True
                    break
        return False

    def title(self, event):
        return f"Microsoft365: MFA Removed on [{event.get('ObjectId', '')}]"

    def alert_context(self, event):
        return m365_alert_context(event)
